#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for iosxr_logging_global
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: iosxr_logging_global
version_added: 2.4.0
short_description: Resource module to configure logging.
description: This module manages the logging attributes of Cisco IOSXR network devices
notes:
- Tested against IOSXR 7.0.2.
- This module works with connection C(network_cli).
author: Ashwini Mhatre (@amhatre)
options:
  config:
    description: A dictionary of logging options.
    type: dict
    suboptions:
      archive:
        description: logging to a persistent device(disk/harddisk)
        type: dict
        suboptions:
          device:
            type: str
            description: Configure the archive device
          archive_length:
            type: int
            description: The maximum no of weeks of log to maintain.
          archive_size:
            type: int
            description: The total size of the archive.
          file_size:
            type: int
            description: The maximum file size for a single log file..
          frequency:
            type: str
            description: The collection interval for logs.
            choices: ["daily", "weekly"]
          severity: &severity
            description: Logging severity level
            type: str
            choices:
              - alerts
              - critical
              - debugging
              - emergencies
              - errors
              - informational
              - notifications
              - warnings
          threshold:
            type: int
            description: Threshold percent <1-99>.
      buffered:
        description: Set buffered logging parameters
        type: dict
        suboptions:
          size: &size
            description: Logging buffer size
            type: int
          severity: *severity
          discriminator: &discriminator
            description: Establish MD-Buffer association
            type: list
            elements: dict
            suboptions:
              match_params:
                type: str
                description: Set match/no-match discriminator.
                choices: ["match1", "match2", "match3", "nomatch1", "nomatch2", "nomatch3"]
              name:
                type: str
                description: discriminator name.
      console:
        description: Set console logging parameters
        type: dict
        suboptions:
          state: &state
            description: Enable or disable logging.
            type: str
            choices: [ "enabled", "disabled" ]
          severity: &severity1
            description: Logging severity level
            type: str
            choices:
              - alerts
              - critical
              - debugging
              - emergencies
              - errors
              - informational
              - notifications
              - warning
          discriminator: *discriminator
      correlator:
        description: Configure properties of the event correlator
        type: dict
        suboptions:
          buffer_size:
            type: int
            description: Configure size of the correlator buffer.
          rules:
            type: list
            elements: dict
            description: Configure a specified correlation rule.
            suboptions:
              rule_name:
                type: str
                description: name of rule.
              rule_type:
                type: str
                choices: ["stateful", "nonstateful"]
                description: type of rule - stateful or nonstateful.
              timeout:
                type: int
                description: Specify timeout.
              timeout_rootcause:
                type: int
                description: Specify timeout for root-cause.
              context_correlation:
                type: bool
                description: Specify enable correlation on context.
              reissue_nonbistate:
                type: bool
                description: Specify reissue of non-bistate alarms on parent clear.This option is allowed for the rules whose type is stateful.
              reparent:
                type: bool
                description: Specify reparent of alarm on parent clear.This option is allowed for the rules whose type is stateful.
          rule_sets:
            type: list
            elements: dict
            description: Configure a specified correlation ruleset.
            suboptions:
              name:
                type: str
                description: Name of the ruleset
              rulename:
                type: list
                elements: str
                description: Name of the rule
      events:
        type: dict
        description: Configure event monitoring parameters.
        suboptions:
          buffer_size:
            type: int
            description: Set size of the local event buffer.
          display_location:
            type: bool
            description: Include alarm source location in message text.
          filter_match:
            type: list
            elements: str
            description: Configure filter.
          severity: *severity
          threshold:
            type: int
            description: Capacity alarm threshold.
      facility:
        description: Facility parameter for syslog messages
        type: str
        choices:
          - auth
          - cron
          - daemon
          - kern
          - local0
          - local1
          - local2
          - local3
          - local4
          - local5
          - local6
          - local7
          - lpr
          - mail
          - news
          - sys10
          - sys11
          - sys12
          - sys13
          - sys14
          - sys9
          - syslog
          - user
          - uucp
      files:
        type: list
        elements: dict
        description: Set file logging.
        suboptions:
          name:
            description: name of file.
            type: str
          path:
            description: Set file path.
            type: str
          maxfilesize:
            type: int
            description: Set max file size.
          severity:
            description: Logging severity level
            type: str
            choices:
              - alerts
              - critical
              - debugging
              - emergencies
              - errors
              - info
              - notifications
              - warning
      format:
        type: bool
        description: Enable to send the syslog message rfc5424 format .
      history:
        description: Configure syslog history table
        type: dict
        suboptions:
          state: *state
          size: *size
          severity:
            description: Logging severity level
            type: str
            choices:
              - alerts
              - critical
              - debugging
              - emergencies
              - errors
              - informational
              - notifications
              - warnings
      hostnameprefix:
        type: str
        description: Hostname prefix to add on msgs to servers.
      ipv4: &ip
        description: Mark the dscp/precedence bit for ipv4 packets.
        type: dict
        suboptions:
          dscp:
            description: Set IP DSCP (DiffServ CodePoint).Please refer vendor document for valid entries.
            type: str
          precedence:
            description: Set precedence Please refer vendor document for valid entries.
            type: str
      ipv6: *ip
      localfilesize:
        type: int
        description: Set size of the local log file
      monitor:
        description: Set terminal line (monitor) logging parameters
        type: dict
        suboptions:
          state: *state
          discriminator: *discriminator
          severity: *severity1
      source_interfaces:
        description: Specify interface for source address in logging transactions
        type: list
        elements: dict
        suboptions:
          interface:
            description: Interface name with number
            type: str
          vrf:
            description: VPN Routing/Forwarding instance name
            type: str
      suppress:
        type: dict
        description: Suppress logging behaviour.
        suboptions:
          apply_rule:
            type: str
            description: Apply suppression rule.
          duplicates:
            type: bool
            description: Suppress consecutive duplicate messages.
      tls_servers:
        type: list
        elements: dict
        description: Secure server over tls.
        suboptions:
          name:
            type: str
            description: Name for the tls peer configuration.
          severity: *severity
          tls_hostname:
            type: str
            description: Name of the logging host.
          trustpoint:
            type: str
            description: Name of the trustpoint configured.
          vrf:
            type: str
            description: name of vrf.
      trap:
        description: Set syslog server logging level
        type: dict
        suboptions:
          state: *state
          severity: *severity1
      hosts:
        description: Set syslog server IP address and parameters
        type: list
        elements: dict
        suboptions:
          severity:
            description: Logging severity level
            type: str
            choices:
              - alerts
              - critical
              - debugging
              - emergencies
              - error
              - info
              - notifications
              - warning
          host:
            description: IPv4/Ipv6 address or hostname of the syslog server
            type: str
          port:
            description: Set <0-65535>  non-default Port.
            type: str
            default: default
          vrf:
            description: Set VRF option
            type: str
            default: default
  running_config:
    description:
      - This option is used only with state I(parsed).
      - The value of this option should be the output received from the IOS device by
        executing the command B(show running-config | include logging).
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into Ansible structured data as per the resource module's argspec
        and the value is then returned in the I(parsed) key within the result.
    type: str
  state:
    choices:
      - merged
      - replaced
      - overridden
      - deleted
      - gathered
      - parsed
      - rendered
    default: merged
    description:
      - The state the configuration should be left in
    type: str
"""
EXAMPLES = """
# Using merged
# -----------------
# Before state
# RP/0/0/CPU0:10#show running-config logging
# Thu Feb  4 09:38:36.245 UTC
# % No such configuration item(s)
# RP/0/0/CPU0:10#
#
#
- name: Merge the provided configuration with the existing running configuration
  cisco.iosxr.iosxr_logging_global:
    config:
      buffered:
        size: 2097152
        severity: warnings
      correlator:
        buffer_size: 1024
      events:
        display_location: true
      files:
        - maxfilesize: '1024'
          name: test
          path: test
          severity: info
      hostnameprefix: test
      hosts:
        - host: 1.1.1.1
          port: default
          severity: critical
          vrf: default
      ipv4:
        dscp: af11
      localfilesize: 1024
      monitor:
        severity: errors
      source_interfaces:
        - interface: GigabitEthernet0/0/0/0
          vrf: test
      tls_servers:
        - name: test
          tls_hostname: test2
          trustpoint: test2
          vrf: test
      trap:
        severity: informational
    state: merged
#
#
# After state:
# -------------------------------------------
# RP/0/0/CPU0:10#show running-config logging
# Tue Jul 20 18:09:18.491 UTC
# logging tls-server test
#  vrf test
#  trustpoint test2
#  tls-hostname test2
# !
# logging file test path test maxfilesize 1024 severity info
# logging ipv4 dscp af11
# logging trap informational
# logging events display-location
# logging monitor errors
# logging buffered 2097152
# logging buffered warnings
# logging 1.1.1.1 vrf default severity critical port default
# logging correlator buffer-size 1024
# logging localfilesize 1024
# logging source-interface GigabitEthernet0/0/0/0 vrf test
# logging hostnameprefix test
# ------------------------------------------------
# Module execution
#
#     "after": {
#         "buffered": {
#             "severity": "errors"
#         },
#         "correlator": {
#             "buffer_size": 1024
#         },
#         "files": [
#             {
#                 "maxfilesize": "1024",
#                 "name": "test",
#                 "path": "test1",
#                 "severity": "info"
#             }
#         ],
#         "hostnameprefix": "test1",
#         "hosts": [
#             {
#                 "host": "1.1.1.3",
#                 "port": "default",
#                 "severity": "critical",
#                 "vrf": "default"
#             }
#         ],
#         "ipv6": {
#             "dscp": "af11"
#         },
#         "localfilesize": 1024,
#         "source_interfaces": [
#             {
#                 "interface": "GigabitEthernet0/0/0/0",
#                 "vrf": "test1"
#             }
#         ],
#         "tls_servers": [
#             {
#                 "name": "test",
#                 "tls_hostname": "test2",
#                 "trustpoint": "test",
#                 "vrf": "test"
#             }
#         ]
#     },
#     "before": {},
#     "changed": true,
#     "commands": [
#         "logging buffered errors",
#         "logging correlator buffer-size 1024",
#         "logging hostnameprefix test1",
#         "logging ipv6 dscp af11",
#         "logging localfilesize 1024",
#         "logging trap disable",
#         "logging monitor disable",
#         "logging history disable",
#         "logging console disable",
#         "logging 1.1.1.3 vrf default severity critical port default",
#         "logging file test path test1 maxfilesize 1024 severity info",
#         "logging source-interface GigabitEthernet0/0/0/0 vrf test1",
#         "logging tls-server test tls-hostname test2",
#         "logging tls-server test trustpoint test",
#         "logging tls-server test vrf test"
#     ],
#     "invocation": {
#         "module_args": {
#             "config": {
#                 "archive": null,
#                 "buffered": {
#                     "discriminator": null,
#                     "severity": "errors",
#                     "size": null
#                 },
#                 "console": {
#                     "discriminator": null,
#                     "severity": null,
#                     "state": "disabled"
#                 },
#                 "correlator": {
#                     "buffer_size": 1024,
#                     "rule_set": null,
#                     "rules": null
#                 },
#                 "events": null,
#                 "facility": null,
#                 "files": [
#                     {
#                         "maxfilesize": "1024",
#                         "name": "test",
#                         "path": "test1",
#                         "severity": "info"
#                     }
#                 ],
#                 "format": null,
#                 "history": {
#                     "severity": null,
#                     "size": null,
#                     "state": "disabled"
#                 },
#                 "hostnameprefix": "test1",
#                 "hosts": [
#                     {
#                         "host": "1.1.1.3",
#                         "port": "default",
#                         "severity": "critical",
#                         "vrf": "default"
#                     }
#                 ],
#                 "ipv4": null,
#                 "ipv6": {
#                     "dscp": "af11",
#                     "precedence": null
#                 },
#                 "localfilesize": 1024,
#                 "monitor": {
#                     "discriminator": null,
#                     "severity": null,
#                     "state": "disabled"
#                 },
#                 "source_interfaces": [
#                     {
#                         "interface": "GigabitEthernet0/0/0/0",
#                         "vrf": "test1"
#                     }
#                 ],
#                 "suppress": null,
#                 "tls_servers": [
#                     {
#                         "name": "test",
#                         "severity": null,
#                         "tls_hostname": "test2",
#                         "trustpoint": "test",
#                         "vrf": "test"
#                     }
#                 ],
#                 "trap": {
#                     "severity": null,
#                     "state": "disabled"
#                 }
#             },
#             "running_config": null,
#             "state": "merged"
#         }
#     }
# }
#
# Using replaced:
# -----------------------------------------------------------
#
# Before state
# RP/0/0/CPU0:10#show running-config logging
# Tue Jul 20 18:09:18.491 UTC
# logging tls-server test
#  vrf test
#  trustpoint test2
#  tls-hostname test2
# !
# logging file test path test maxfilesize 1024 severity info
# logging ipv4 dscp af11
# logging trap informational
# logging events display-location
# logging monitor errors
# logging buffered 2097152
# logging buffered warnings
# logging 1.1.1.1 vrf default severity critical port default
# logging correlator buffer-size 1024
# logging localfilesize 1024
# logging source-interface GigabitEthernet0/0/0/0 vrf test
# logging hostnameprefix test
# -----------------------------------------------------------
#
- name: Replace BGP configuration with provided configuration
  cisco.iosxr.iosxr_logging_global:
    state: replaced
    config:
      buffered:
        severity: errors
      correlator:
        buffer_size: 1024
      files:
        - maxfilesize: '1024'
          name: test
          path: test1
          severity: info
      hostnameprefix: test1
      hosts:
        - host: 1.1.1.3
          port: default
          severity: critical
          vrf: default
      ipv6:
        dscp: af11
      localfilesize: 1024
      monitor:
        severity: errors
      tls_servers:
        - name: test
          tls_hostname: test2
          trustpoint: test
          vrf: test
      trap:
        severity: critical
#
# After state:
# RP/0/0/CPU0:10#show running-config logging
# Tue Jul 20 18:31:51.709 UTC
# logging tls-server test
#  vrf test
#  trustpoint test
#  tls-hostname test2
# !
# logging file test path test1 maxfilesize 1024 severity info
# logging ipv6 dscp af11
# logging trap critical
# logging monitor errors
# logging buffered errors
# logging 1.1.1.3 vrf default severity critical port default
# logging correlator buffer-size 1024
# logging localfilesize 1024
# logging hostnameprefix test1
# -----------------------------------------------------------------
#
# Module Execution:
# "after": {
#         "buffered": {
#             "severity": "errors"
#         },
#         "correlator": {
#             "buffer_size": 1024
#         },
#         "files": [
#             {
#                 "maxfilesize": "1024",
#                 "name": "test",
#                 "path": "test1",
#                 "severity": "info"
#             }
#         ],
#         "hostnameprefix": "test1",
#         "hosts": [
#             {
#                 "host": "1.1.1.3",
#                 "port": "default",
#                 "severity": "critical",
#                 "vrf": "default"
#             }
#         ],
#         "ipv6": {
#             "dscp": "af11"
#         },
#         "localfilesize": 1024,
#         "monitor": {
#             "severity": "errors"
#         },
#         "tls_servers": [
#             {
#                 "name": "test",
#                 "tls_hostname": "test2",
#                 "trustpoint": "test",
#                 "vrf": "test"
#             }
#         ],
#         "trap": {
#             "severity": "critical"
#         }
#     },
#     "before": {
#         "buffered": {
#             "severity": "warnings",
#             "size": 2097152
#         },
#         "correlator": {
#             "buffer_size": 1024
#         },
#         "events": {
#             "display_location": true
#         },
#         "files": [
#             {
#                 "maxfilesize": "1024",
#                 "name": "test",
#                 "path": "test",
#                 "severity": "info"
#             }
#         ],
#         "hostnameprefix": "test",
#         "hosts": [
#             {
#                 "host": "1.1.1.1",
#                 "port": "default",
#                 "severity": "critical",
#                 "vrf": "default"
#             }
#         ],
#         "ipv4": {
#             "dscp": "af11"
#         },
#         "localfilesize": 1024,
#         "monitor": {
#             "severity": "errors"
#         },
#         "source_interfaces": [
#             {
#                 "interface": "GigabitEthernet0/0/0/0",
#                 "vrf": "test"
#             }
#         ],
#         "tls_servers": [
#             {
#                 "name": "test",
#                 "tls_hostname": "test2",
#                 "trustpoint": "test2",
#                 "vrf": "test"
#             }
#         ],
#         "trap": {
#             "severity": "informational"
#         }
#     },
#     "changed": true,
#     "commands": [
#         "no logging buffered 2097152",
#         "no logging events display-location",
#         "no logging ipv4 dscp af11",
#         "no logging 1.1.1.1 vrf default severity critical port default",
#         "no logging source-interface GigabitEthernet0/0/0/0 vrf test",
#         "logging buffered errors",
#         "logging hostnameprefix test1",
#         "logging ipv6 dscp af11",
#         "logging trap critical",
#         "logging 1.1.1.3 vrf default severity critical port default",
#         "logging file test path test1 maxfilesize 1024 severity info",
#         "logging tls-server test trustpoint test"
#     ],
#
#
#
# Using deleted:
# -----------------------------------------------------------
# Before state:
# RP/0/0/CPU0:10#show running-config logging
# Tue Jul 20 18:09:18.491 UTC
# logging tls-server test
#  vrf test
#  trustpoint test2
#  tls-hostname test2
# !
# logging file test path test maxfilesize 1024 severity info
# logging ipv4 dscp af11
# logging trap informational
# logging events display-location
# logging monitor errors
# logging buffered 2097152
# logging buffered warnings
# logging 1.1.1.1 vrf default severity critical port default
# logging correlator buffer-size 1024
# logging localfilesize 1024
# logging source-interface GigabitEthernet0/0/0/0 vrf test
# logging hostnameprefix test
#
# -----------------------------------------------------------
- name: Delete given logging_global configuration
  cisco.iosxr.iosxr_logging_global:
    state: deleted
#
# After state:
# RP/0/0/CPU0:10#show running-config
#
# -------------------------------------------------------------
# Module Execution:
#
# "after": {},
#     "before": {
#         "buffered": {
#             "severity": "warnings",
#             "size": 2097152
#         },
#         "correlator": {
#             "buffer_size": 1024
#         },
#         "events": {
#             "display_location": true
#         },
#         "files": [
#             {
#                 "maxfilesize": "1024",
#                 "name": "test",
#                 "path": "test",
#                 "severity": "info"
#             }
#         ],
#         "hostnameprefix": "test",
#         "hosts": [
#             {
#                 "host": "1.1.1.1",
#                 "port": "default",
#                 "severity": "critical",
#                 "vrf": "default"
#             }
#         ],
#         "ipv4": {
#             "dscp": "af11"
#         },
#         "localfilesize": 1024,
#         "monitor": {
#             "severity": "errors"
#         },
#         "source_interfaces": [
#             {
#                 "interface": "GigabitEthernet0/0/0/0",
#                 "vrf": "test"
#             }
#         ],
#         "tls_servers": [
#             {
#                 "name": "test",
#                 "tls_hostname": "test2",
#                 "trustpoint": "test2",
#                 "vrf": "test"
#             }
#         ],
#         "trap": {
#             "severity": "informational"
#         }
#     },
#     "changed": true,
#     "commands": [
#         "no logging buffered 2097152",
#         "no logging buffered warnings",
#         "no logging correlator buffer-size 1024",
#         "no logging events display-location",
#         "no logging hostnameprefix test",
#         "no logging ipv4 dscp af11",
#         "no logging localfilesize 1024",
#         "no logging monitor errors",
#         "no logging trap informational",
#         "no logging 1.1.1.1 vrf default severity critical port default",
#         "no logging file test path test maxfilesize 1024 severity info",
#         "no logging source-interface GigabitEthernet0/0/0/0 vrf test",
#         "no logging tls-server test"
#     ],
#     "invocation": {
#         "module_args": {
#             "config": null,
#             "running_config": null,
#             "state": "deleted"
#         }
#     }
#
#
#
# using gathered:
# ------------------------------------------------------------
# Before state:
# RP/0/0/CPU0:10#show running-config logging
# Tue Jul 20 18:09:18.491 UTC
# logging tls-server test
#  vrf test
#  trustpoint test2
#  tls-hostname test2
# !
# logging file test path test maxfilesize 1024 severity info
# logging ipv4 dscp af11
# logging trap informational
# logging events display-location
# logging monitor errors
# logging buffered 2097152
# logging buffered warnings
# logging 1.1.1.1 vrf default severity critical port default
# logging correlator buffer-size 1024
# logging localfilesize 1024
# logging source-interface GigabitEthernet0/0/0/0 vrf test
# logging hostnameprefix test
#
#
- name: Gather iosxr_logging_global facts using gathered state
  cisco.iosxr.iosxr_logging_global:
    state: gathered
#
# -------------------------------------------------------------
# Module Execution:
#
# "changed": false,
# "gathered": {
#         "buffered": {
#             "severity": "warnings",
#             "size": 2097152
#         },
#         "correlator": {
#             "buffer_size": 1024
#         },
#         "events": {
#             "display_location": true
#         },
#         "files": [
#             {
#                 "maxfilesize": "1024",
#                 "name": "test",
#                 "path": "test",
#                 "severity": "info"
#             }
#         ],
#         "hostnameprefix": "test",
#         "hosts": [
#             {
#                 "host": "1.1.1.1",
#                 "port": "default",
#                 "severity": "critical",
#                 "vrf": "default"
#             }
#         ],
#         "ipv4": {
#             "dscp": "af11"
#         },
#         "localfilesize": 1024,
#         "monitor": {
#             "severity": "errors"
#         },
#         "source_interfaces": [
#             {
#                 "interface": "GigabitEthernet0/0/0/0",
#                 "vrf": "test"
#             }
#         ],
#         "tls_servers": [
#             {
#                 "name": "test",
#                 "tls_hostname": "test2",
#                 "trustpoint": "test2",
#                 "vrf": "test"
#             }
#         ],
#         "trap": {
#             "severity": "informational"
#         }
#     },
#     "invocation": {
#         "module_args": {
#             "config": null,
#             "running_config": null,
#             "state": "gathered"
#         }
# }
#
#
# Using parsed:
# ---------------------------------------------------------------
#
# parsed.cfg
#
# logging tls-server test
#  vrf test
#  trustpoint test2
#  tls-hostname test2
# !
# logging file test path test maxfilesize 1024 severity info
# logging ipv4 dscp af11
# logging trap informational
# logging events display-location
# logging monitor errors
# logging buffered 2097152
# logging buffered warnings
# logging 1.1.1.1 vrf default severity critical port default
# logging correlator buffer-size 1024
# logging localfilesize 1024
# logging source-interface GigabitEthernet0/0/0/0 vrf test
# logging hostnameprefix test
#
#
- name: Parse externally provided Logging global config to agnostic model
  cisco.iosxr.iosxr_logging_global:
    running_config: "{{ lookup('file', './fixtures/parsed.cfg') }}"
    state: parsed
# ----------------------------------------------------------------
# Module execution:
# "changed": false,
# "parsed": {
#         "buffered": {
#             "severity": "warnings",
#             "size": 2097152
#         },
#         "correlator": {
#             "buffer_size": 1024
#         },
#         "events": {
#             "display_location": true
#         },
#         "files": [
#             {
#                 "maxfilesize": "1024",
#                 "name": "test",
#                 "path": "test",
#                 "severity": "info"
#             }
#         ],
#         "hostnameprefix": "test",
#         "hosts": [
#             {
#                 "host": "1.1.1.1",
#                 "port": "default",
#                 "severity": "critical",
#                 "vrf": "default"
#             }
#         ],
#         "ipv4": {
#             "dscp": "af11"
#         },
#         "localfilesize": 1024,
#         "monitor": {
#             "severity": "errors"
#         },
#         "source_interfaces": [
#             {
#                 "interface": "GigabitEthernet0/0/0/0",
#                 "vrf": "test"
#             }
#         ],
#         "tls_servers": [
#             {
#                 "name": "test",
#                 "tls_hostname": "test2",
#                 "trustpoint": "test2",
#                 "vrf": "test"
#             }
#         ],
#         "trap": {
#             "severity": "informational"
#         }
#     }
#
#
# Using rendered:
# ----------------------------------------------------------------------------
- name: >-
    Render platform specific configuration lines with state rendered (without
    connecting to the device)
  cisco.iosxr.iosxr_logging_global:
    state: rendered
    config:
      buffered:
        size: 2097152
        severity: warnings
      correlator:
        buffer_size: 1024
      events:
        display_location: true
      files:
        - maxfilesize: '1024'
          name: test
          path: test
          severity: info
      hostnameprefix: test
      hosts:
        - host: 1.1.1.1
          port: default
          severity: critical
          vrf: default
      ipv4:
        dscp: af11
      localfilesize: 1024
      monitor:
        severity: errors
      source_interfaces:
        - interface: GigabitEthernet0/0/0/0
          vrf: test
      tls_servers:
        - name: test
          tls_hostname: test2
          trustpoint: test2
          vrf: test
      trap:
        severity: informational
# ----------------------------------------------------------------
# Module Execution:
# "rendered": [
#         "logging buffered errors",
#         "logging correlator buffer-size 1024",
#         "logging hostnameprefix test1",
#         "logging ipv6 dscp af11",
#         "logging localfilesize 1024",
#         "logging trap disable",
#         "logging monitor disable",
#         "logging history disable",
#         "logging console disable",
#         "logging 1.1.1.3 vrf default severity critical port default",
#         "logging file test path test1 maxfilesize 1024 severity info",
#         "logging source-interface GigabitEthernet0/0/0/0 vrf test1",
#         "logging tls-server test tls-hostname test2",
#         "logging tls-server test trustpoint test",
#         "logging tls-server test vrf test"
#     ]
#
# Using overridden:
# ---------------------------------------------------------------------------------
# Before state:
# RP/0/0/CPU0:10#show running-config logging
# Tue Jul 20 18:09:18.491 UTC
# logging tls-server test
#  vrf test
#  trustpoint test2
#  tls-hostname test2
# !
# logging file test path test maxfilesize 1024 severity info
# logging ipv4 dscp af11
# logging trap informational
# logging events display-location
# logging monitor errors
# logging buffered 2097152
# logging buffered warnings
# logging 1.1.1.1 vrf default severity critical port default
# logging correlator buffer-size 1024
# logging localfilesize 1024
# logging source-interface GigabitEthernet0/0/0/0 vrf test
# logging hostnameprefix test
#
# -----------------------------------------------------------
#
- name: Overridde logging global configuration with provided configuration
  cisco.iosxr.iosxr_logging_global:
    state: overridden
    config:
      buffered:
        severity: errors
      correlator:
        buffer_size: 1024
      files:
        - maxfilesize: '1024'
          name: test
          path: test1
          severity: info
      hostnameprefix: test1
      hosts:
        - host: 1.1.1.3
          port: default
          severity: critical
          vrf: default
      ipv6:
        dscp: af11
      localfilesize: 1024
      monitor:
        severity: errors
      tls_servers:
        - name: test
          tls_hostname: test2
          trustpoint: test
          vrf: test
      trap:
        severity: critical
#
# After state:
# RP/0/0/CPU0:10#show running-config logging
# Tue Jul 20 18:31:51.709 UTC
# logging tls-server test
#  vrf test
#  trustpoint test
#  tls-hostname test2
# !
# logging file test path test1 maxfilesize 1024 severity info
# logging ipv6 dscp af11
# logging trap critical
# logging monitor errors
# logging buffered errors
# logging 1.1.1.3 vrf default severity critical port default
# logging correlator buffer-size 1024
# logging localfilesize 1024
# logging hostnameprefix test1
# -----------------------------------------------------------------
#
# Module Execution:
# "after": {
#         "buffered": {
#             "severity": "errors"
#         },
#         "correlator": {
#             "buffer_size": 1024
#         },
#         "files": [
#             {
#                 "maxfilesize": "1024",
#                 "name": "test",
#                 "path": "test1",
#                 "severity": "info"
#             }
#         ],
#         "hostnameprefix": "test1",
#         "hosts": [
#             {
#                 "host": "1.1.1.3",
#                 "port": "default",
#                 "severity": "critical",
#                 "vrf": "default"
#             }
#         ],
#         "ipv6": {
#             "dscp": "af11"
#         },
#         "localfilesize": 1024,
#         "monitor": {
#             "severity": "errors"
#         },
#         "tls_servers": [
#             {
#                 "name": "test",
#                 "tls_hostname": "test2",
#                 "trustpoint": "test",
#                 "vrf": "test"
#             }
#         ],
#         "trap": {
#             "severity": "critical"
#         }
#     },
#     "before": {
#         "buffered": {
#             "severity": "warnings",
#             "size": 2097152
#         },
#         "correlator": {
#             "buffer_size": 1024
#         },
#         "events": {
#             "display_location": true
#         },
#         "files": [
#             {
#                 "maxfilesize": "1024",
#                 "name": "test",
#                 "path": "test",
#                 "severity": "info"
#             }
#         ],
#         "hostnameprefix": "test",
#         "hosts": [
#             {
#                 "host": "1.1.1.1",
#                 "port": "default",
#                 "severity": "critical",
#                 "vrf": "default"
#             }
#         ],
#         "ipv4": {
#             "dscp": "af11"
#         },
#         "localfilesize": 1024,
#         "monitor": {
#             "severity": "errors"
#         },
#         "source_interfaces": [
#             {
#                 "interface": "GigabitEthernet0/0/0/0",
#                 "vrf": "test"
#             }
#         ],
#         "tls_servers": [
#             {
#                 "name": "test",
#                 "tls_hostname": "test2",
#                 "trustpoint": "test2",
#                 "vrf": "test"
#             }
#         ],
#         "trap": {
#             "severity": "informational"
#         }
#     },
#     "changed": true,
#     "commands": [
#         "no logging buffered 2097152",
#         "no logging events display-location",
#         "no logging ipv4 dscp af11",
#         "no logging 1.1.1.1 vrf default severity critical port default",
#         "no logging source-interface GigabitEthernet0/0/0/0 vrf test",
#         "logging buffered errors",
#         "logging hostnameprefix test1",
#         "logging ipv6 dscp af11",
#         "logging trap critical",
#         "logging 1.1.1.3 vrf default severity critical port default",
#         "logging file test path test1 maxfilesize 1024 severity info",
#         "logging tls-server test trustpoint test"
#     ],
#
"""

RETURN = """
before:
  description: The configuration prior to the module execution.
  returned: when state is I(merged), I(replaced), I(overridden), I(deleted) or I(purged)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
after:
  description: The resulting configuration after module execution.
  returned: when changed
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
commands:
  description: The set of commands pushed to the remote device.
  returned: when state is I(merged), I(replaced), I(overridden), I(deleted) or I(purged)
  type: list
  sample:
    - "logging file test path test1 maxfilesize 1024 severity info"
    - "logging ipv6 dscp af11"
    - "logging trap critical"
    - "logging monitor errors"
    - "logging buffered errors"
    - "logging 1.1.1.3 vrf default severity critical port default"
rendered:
  description: The provided configuration in the task rendered in device-native format (offline).
  returned: when state is I(rendered)
  type: list
  sample:
    - "logging buffered errors"
    - "logging correlator buffer-size 1024"
    - "logging hostnameprefix test1"
    - "logging ipv6 dscp af11"
    - "logging localfilesize 1024"
    - "logging trap disable"
    - "logging monitor disable"
    - "logging history disable"
    - "logging console disable"
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when state is I(gathered)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when state is I(parsed)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.argspec.logging_global.logging_global import (
    Logging_globalArgs,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.config.logging_global.logging_global import (
    Logging_global,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Logging_globalArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Logging_global(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: guest_smtp_notification_settings
short_description: Resource module for Guest SMTP Notification Settings
description:
- Manage operations create and update of the resource Guest SMTP Notification Settings.
- This API creates a guest SMTP notification configuration.
- This API allows the client to update a SMTP configuration setting.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  connectionTimeout:
    description: Interval in seconds for all the SMTP client connections.
    type: str
  defaultFromAddress:
    description: The default from email address to be used to send emails from.
    type: str
  id:
    description: Guest SMTP Notification Settings's id.
    type: str
  notificationEnabled:
    description: Indicates if the email notification service is to be enabled.
    type: bool
  password:
    description: Password of Secure SMTP server.
    type: str
  smtpPort:
    description: Port at which SMTP Secure Server is listening.
    type: str
  smtpServer:
    description: The SMTP server ip address or fqdn such as outbound.mycompany.com.
    type: str
  useDefaultFromAddress:
    description: If the default from address should be used rather than using a sponsor
      user email address.
    type: bool
  usePasswordAuthentication:
    description: If configured to true, SMTP server authentication will happen using
      username/password.
    type: bool
  useTLSorSSLEncryption:
    description: If configured to true, SMTP server authentication will happen using
      TLS/SSL.
    type: bool
  userName:
    description: Username of Secure SMTP server.
    type: str
requirements:
- ciscoisesdk >= 2.1.1
- python >= 3.5
notes:
  - SDK Method used are
    guest_smtp_notification_configuration.GuestSmtpNotificationConfiguration.create_guest_smtp_notification_settings,
    guest_smtp_notification_configuration.GuestSmtpNotificationConfiguration.update_guest_smtp_notification_settings_by_id,

  - Paths used are
    post /ers/config/guestsmtpnotificationsettings,
    put /ers/config/guestsmtpnotificationsettings/{id},

"""

EXAMPLES = r"""
- name: Update by id
  cisco.ise.guest_smtp_notification_settings:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    connectionTimeout: string
    defaultFromAddress: string
    id: string
    notificationEnabled: true
    password: string
    smtpPort: string
    smtpServer: string
    useDefaultFromAddress: true
    usePasswordAuthentication: true
    useTLSorSSLEncryption: true
    userName: string

- name: Create
  cisco.ise.guest_smtp_notification_settings:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    connectionTimeout: string
    defaultFromAddress: string
    notificationEnabled: true
    password: string
    smtpPort: string
    smtpServer: string
    useDefaultFromAddress: true
    usePasswordAuthentication: true
    useTLSorSSLEncryption: true
    userName: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "smtpServer": "string",
      "notificationEnabled": true,
      "useDefaultFromAddress": true,
      "defaultFromAddress": "string",
      "smtpPort": "string",
      "connectionTimeout": "string",
      "useTLSorSSLEncryption": true,
      "usePasswordAuthentication": true,
      "userName": "string",
      "password": "string",
      "link": {
        "rel": "string",
        "href": "string",
        "type": "string"
      }
    }

ise_update_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  version_added: '1.1.0'
  type: dict
  sample: >
    {
      "UpdatedFieldsList": {
        "updatedField": [
          {
            "field": "string",
            "oldValue": "string",
            "newValue": "string"
          }
        ],
        "field": "string",
        "oldValue": "string",
        "newValue": "string"
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sda_virtual_network_v2
short_description: Resource module for Sda Virtual Network V2
description:
- Manage operations create, update and delete of the resource Sda Virtual Network V2.
- Add virtual network with scalable groups at global level.
- Delete virtual network with scalable groups.
- Update virtual network with scalable groups.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  isGuestVirtualNetwork:
    description: Guest Virtual Network enablement flag, default value is False.
    type: bool
  scalableGroupNames:
    description: Scalable Group to be associated to virtual network.
    elements: str
    type: list
  vManageVpnId:
    description: VManage vpn id for SD-WAN.
    type: str
  virtualNetworkName:
    description: Virtual Network Name to be assigned at global level.
    type: str
requirements:
- dnacentersdk >= 2.5.5
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for SDA AddVirtualNetworkWithScalableGroups
  description: Complete reference of the AddVirtualNetworkWithScalableGroups API.
  link: https://developer.cisco.com/docs/dna-center/#!add-virtual-network-with-scalable-groups
- name: Cisco DNA Center documentation for SDA DeleteVirtualNetworkWithScalableGroups
  description: Complete reference of the DeleteVirtualNetworkWithScalableGroups API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-virtual-network-with-scalable-groups
- name: Cisco DNA Center documentation for SDA UpdateVirtualNetworkWithScalableGroups
  description: Complete reference of the UpdateVirtualNetworkWithScalableGroups API.
  link: https://developer.cisco.com/docs/dna-center/#!update-virtual-network-with-scalable-groups
notes:
  - SDK Method used are
    sda.Sda.add_virtual_network_with_scalable_groups,
    sda.Sda.delete_virtual_network_with_scalable_groups,
    sda.Sda.update_virtual_network_with_scalable_groups,

  - Paths used are
    post /dna/intent/api/v1/virtual-network,
    delete /dna/intent/api/v1/virtual-network,
    put /dna/intent/api/v1/virtual-network,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.sda_virtual_network_v2:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    isGuestVirtualNetwork: true
    scalableGroupNames:
    - string
    vManageVpnId: string
    virtualNetworkName: string

- name: Delete all
  cisco.dnac.sda_virtual_network_v2:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    virtualNetworkName: string

- name: Update all
  cisco.dnac.sda_virtual_network_v2:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    isGuestVirtualNetwork: true
    scalableGroupNames:
    - string
    vManageVpnId: string
    virtualNetworkName: string

"""

RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "status": "string",
      "description": "string",
      "taskId": "string",
      "taskStatusUrl": "string",
      "executionStatusUrl": "string",
      "executionId": "string"
    }
"""



# -*- coding: utf-8 -*-
# This code is part of the Ansible collection community.docker, but is an independent component.
# This particular file, and this file only, is based on the Docker SDK for Python (https://github.com/docker/docker-py/)
#
# Copyright (c) 2016-2022 Docker, Inc.
#
# It is licensed under the Apache 2.0 license (see LICENSES/Apache-2.0.txt in this collection)
# SPDX-License-Identifier: Apache-2.0

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import json
import logging
import os

from ..constants import IS_WINDOWS_PLATFORM

DOCKER_CONFIG_FILENAME = os.path.join('.docker', 'config.json')
LEGACY_DOCKER_CONFIG_FILENAME = '.dockercfg'

log = logging.getLogger(__name__)


def find_config_file(config_path=None):
    paths = list(filter(None, [
        config_path,  # 1
        config_path_from_environment(),  # 2
        os.path.join(home_dir(), DOCKER_CONFIG_FILENAME),  # 3
        os.path.join(home_dir(), LEGACY_DOCKER_CONFIG_FILENAME),  # 4
    ]))

    log.debug("Trying paths: %s", repr(paths))

    for path in paths:
        if os.path.exists(path):
            log.debug("Found file at path: %s", path)
            return path

    log.debug("No config file found")

    return None


def config_path_from_environment():
    config_dir = os.environ.get('DOCKER_CONFIG')
    if not config_dir:
        return None
    return os.path.join(config_dir, os.path.basename(DOCKER_CONFIG_FILENAME))


def home_dir():
    """
    Get the user's home directory, using the same logic as the Docker Engine
    client - use %USERPROFILE% on Windows, $HOME/getuid on POSIX.
    """
    if IS_WINDOWS_PLATFORM:
        return os.environ.get('USERPROFILE', '')
    else:
        return os.path.expanduser('~')


def load_general_config(config_path=None):
    config_file = find_config_file(config_path)

    if not config_file:
        return {}

    try:
        with open(config_file) as f:
            return json.load(f)
    except (IOError, ValueError) as e:
        # In the case of a legacy `.dockercfg` file, we won't
        # be able to load any JSON data.
        log.debug(e)

    log.debug("All parsing attempts failed - returning empty config")
    return {}



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: devices_wireless_radio_settings
short_description: Resource module for devices _wireless _radio _settings
description:
- Manage operation update of the resource devices _wireless _radio _settings.
- Update the radio settings of a device.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  fiveGhzSettings:
    description: Manual radio settings for 5 GHz.
    suboptions:
      channel:
        description: Sets a manual channel for 5 GHz. Can be '36', '40', '44', '48',
          '52', '56', '60', '64', '100', '104', '108', '112', '116', '120', '124', '128',
          '132', '136', '140', '144', '149', '153', '157', '161', '165', '169', '173'
          or '177' or null for using auto channel.
        type: int
      channelWidth:
        description: Sets a manual channel for 5 GHz. Can be '0', '20', '40', '80' or
          '160' or null for using auto channel width.
        type: int
      targetPower:
        description: Set a manual target power for 5 GHz. Can be between '8' or '30'
          or null for using auto power range.
        type: int
    type: dict
  rfProfileId:
    description: The ID of an RF profile to assign to the device. If the value of this
      parameter is null, the appropriate basic RF profile (indoor or outdoor) will be
      assigned to the device. Assigning an RF profile will clear ALL manually configured
      overrides on the device (channel width, channel, power).
    type: str
  serial:
    description: Serial path parameter.
    type: str
  twoFourGhzSettings:
    description: Manual radio settings for 2.4 GHz.
    suboptions:
      channel:
        description: Sets a manual channel for 2.4 GHz. Can be '1', '2', '3', '4', '5',
          '6', '7', '8', '9', '10', '11', '12', '13' or '14' or null for using auto
          channel.
        type: int
      targetPower:
        description: Set a manual target power for 2.4 GHz. Can be between '5' or '30'
          or null for using auto power range.
        type: int
    type: dict
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for devices updateDeviceWirelessRadioSettings
  description: Complete reference of the updateDeviceWirelessRadioSettings API.
  link: https://developer.cisco.com/meraki/api-v1/#!update-device-wireless-radio-settings
notes:
  - SDK Method used are
    devices.Devices.update_device_wireless_radio_settings,

  - Paths used are
    put /devices/{serial}/wireless/radio/settings,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.devices_wireless_radio_settings:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    fiveGhzSettings:
      channel: 149
      channelWidth: 20
      targetPower: 15
    rfProfileId: '1234'
    serial: string
    twoFourGhzSettings:
      channel: 11
      targetPower: 21

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: nxos_vrf_interface
extends_documentation_fragment:
- cisco.nxos.nxos
short_description: Manages interface specific VRF configuration.
description:
- Manages interface specific VRF configuration.
version_added: 1.0.0
author:
- Jason Edelman (@jedelman8)
- Gabriele Gerbino (@GGabriele)
notes:
- Tested against NXOSv 7.3.(0)D1(1) on VIRL
- Unsupported for Cisco MDS
- VRF needs to be added globally with M(cisco.nxos.nxos_vrf) before adding a VRF to an interface.
- Remove a VRF from an interface will still remove all L3 attributes just as it does
  from CLI.
- VRF is not read from an interface until IP address is configured on that interface.
options:
  vrf:
    description:
    - Name of VRF to be managed.
    required: true
    type: str
  interface:
    description:
    - Full name of interface to be managed, i.e. Ethernet1/1.
    required: true
    type: str
  state:
    description:
    - Manages desired state of the resource.
    required: false
    default: present
    choices:
    - present
    - absent
    type: str
"""

EXAMPLES = """
- name: Ensure vrf ntc exists on Eth1/1
  cisco.nxos.nxos_vrf_interface:
    vrf: ntc
    interface: Ethernet1/1
    state: present

- name: Ensure ntc VRF does not exist on Eth1/1
  cisco.nxos.nxos_vrf_interface:
    vrf: ntc
    interface: Ethernet1/1
    state: absent
"""

RETURN = """
commands:
    description: commands sent to the device
    returned: always
    type: list
    sample: ["interface loopback16", "vrf member ntc"]
"""
import re

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    get_capabilities,
    get_interface_type,
    load_config,
    normalize_interface,
    run_commands,
)


def execute_show_command(command, module):
    if "show run" not in command:
        output = "json"
    else:
        output = "text"
    cmds = [{"command": command, "output": output}]
    return run_commands(module, cmds)[0]


def get_interface_mode(interface, intf_type, module):
    command = "show interface {0}".format(interface)
    interface = {}
    mode = "unknown"

    if intf_type in ["ethernet", "portchannel"]:
        body = execute_show_command(command, module)
        try:
            interface_table = body["TABLE_interface"]["ROW_interface"]
        except KeyError:
            return mode

        if interface_table and isinstance(interface_table, dict):
            mode = str(interface_table.get("eth_mode", "layer3"))
            if mode == "access" or mode == "trunk":
                mode = "layer2"
        else:
            return mode

    elif intf_type == "loopback" or intf_type == "svi":
        mode = "layer3"
    return mode


def get_vrf_list(module):
    command = "show vrf all"
    vrf_list = []
    body = execute_show_command(command, module)

    try:
        vrf_table = body["TABLE_vrf"]["ROW_vrf"]
    except (KeyError, AttributeError):
        return vrf_list

    for each in vrf_table:
        vrf_list.append(str(each["vrf_name"]))

    return vrf_list


def get_interface_info(interface, module):
    if not interface.startswith("loopback"):
        interface = interface.capitalize()

    command = "show run interface {0}".format(interface)
    vrf_regex = r".*vrf\s+member\s+(?P<vrf>\S+).*"

    try:
        body = execute_show_command(command, module)
        match_vrf = re.match(vrf_regex, body, re.DOTALL)
        group_vrf = match_vrf.groupdict()
        vrf = group_vrf["vrf"]
    except (AttributeError, TypeError):
        return ""

    return vrf


def is_default(interface, module):
    command = "show run interface {0}".format(interface)

    try:
        body = execute_show_command(command, module)
        raw_list = body.split("\n")
        if raw_list[-1].startswith("interface"):
            return True
        else:
            return False

    except (KeyError, IndexError):
        return "DNE"


def main():
    argument_spec = dict(
        vrf=dict(required=True),
        interface=dict(type="str", required=True),
        state=dict(default="present", choices=["present", "absent"], required=False),
    )

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    warnings = list()
    results = {"changed": False, "commands": [], "warnings": warnings}

    vrf = module.params["vrf"]
    interface = normalize_interface(module.params["interface"])
    state = module.params["state"]

    device_info = get_capabilities(module)
    network_api = device_info.get("network_api", "nxapi")

    current_vrfs = get_vrf_list(module)
    if vrf not in current_vrfs:
        warnings.append("The VRF is not present/active on the device. Use nxos_vrf to fix this.")

    intf_type = get_interface_type(interface)
    if intf_type != "ethernet" and network_api == "cliconf":
        if is_default(interface, module) == "DNE":
            module.fail_json(
                msg="interface does not exist on switch. Verify "
                "switch platform or create it first with "
                "nxos_interfaces if it's a logical interface",
            )

    mode = get_interface_mode(interface, intf_type, module)
    if mode == "layer2":
        module.fail_json(
            msg="Ensure interface is a Layer 3 port before "
            "configuring a VRF on an interface. You can "
            "use nxos_interfaces",
        )

    current_vrf = get_interface_info(interface, module)
    existing = dict(interface=interface, vrf=current_vrf)
    changed = False

    if not existing["vrf"]:
        pass
    elif vrf != existing["vrf"] and state == "absent":
        module.fail_json(
            msg="The VRF you are trying to remove "
            "from the interface does not exist "
            "on that interface.",
            interface=interface,
            proposed_vrf=vrf,
            existing_vrf=existing["vrf"],
        )

    commands = []
    if existing:
        if state == "absent":
            if existing and vrf == existing["vrf"]:
                command = "no vrf member {0}".format(vrf)
                commands.append(command)

        elif state == "present":
            if existing["vrf"] != vrf:
                command = "vrf member {0}".format(vrf)
                commands.append(command)

    if commands:
        commands.insert(0, "interface {0}".format(interface))

    if commands:
        if module.check_mode:
            module.exit_json(changed=True, commands=commands)
        else:
            load_config(module, commands)
            changed = True
            if "configure" in commands:
                commands.pop(0)

    results["commands"] = commands
    results["changed"] = changed

    module.exit_json(**results)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: networks_appliance_firewall_one_to_many_nat_rules
short_description: Resource module for networks _appliance _firewall _onetomanynatrules
description:
- Manage operation update of the resource networks _appliance _firewall _onetomanynatrules.
- Set the 1 Many NAT mapping rules for an MX network.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  rules:
    description: An array of 1 Many nat rules.
    elements: dict
    suboptions:
      portRules:
        description: An array of associated forwarding rules.
        elements: dict
        suboptions:
          allowedIps:
            description: Remote IP addresses or ranges that are permitted to access
              the internal resource via this port forwarding rule, or 'any'.
            elements: str
            type: list
          localIp:
            description: Local IP address to which traffic will be forwarded.
            type: str
          localPort:
            description: Destination port of the forwarded traffic that will be sent
              from the MX to the specified host on the LAN. If you simply wish to forward
              the traffic without translating the port, this should be the same as the
              Public port.
            type: str
          name:
            description: A description of the rule.
            type: str
          protocol:
            description: '''tcp'' or ''udp''.'
            type: str
          publicPort:
            description: Destination port of the traffic that is arriving on the WAN.
            type: str
        type: list
      publicIp:
        description: The IP address that will be used to access the internal resource
          from the WAN.
        type: str
      uplink:
        description: The physical WAN interface on which the traffic will arrive ('internet1'
          or, if available, 'internet2').
        type: str
    type: list
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for appliance updateNetworkApplianceFirewallOneToManyNatRules
  description: Complete reference of the updateNetworkApplianceFirewallOneToManyNatRules API.
  link: https://developer.cisco.com/meraki/api-v1/#!update-network-appliance-firewall-one-to-many-nat-rules
notes:
  - SDK Method used are
    appliance.Appliance.update_network_appliance_firewall_one_to_many_nat_rules,

  - Paths used are
    put /networks/{networkId}/appliance/firewall/oneToManyNatRules,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.networks_appliance_firewall_one_to_many_nat_rules:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    networkId: string
    rules:
    - portRules:
      - allowedIps:
        - any
        localIp: 192.168.128.1
        localPort: '443'
        name: Rule 1
        protocol: tcp
        publicPort: '9443'
      - allowedIps:
        - 10.82.110.0/24
        - 10.82.111.0/24
        localIp: 192.168.128.1
        localPort: '80'
        name: Rule 2
        protocol: tcp
        publicPort: '8080'
      publicIp: 146.11.11.13
      uplink: internet1

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



# -*- coding: utf-8 -*-
# Copyright (c) Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function
__metaclass__ = type

from ansible.errors import AnsibleFilterError
from ansible.module_utils.common.collections import is_sequence


def remove_duplicates(lst):
    seen = set()
    seen_add = seen.add
    result = []
    for item in lst:
        try:
            if item not in seen:
                seen_add(item)
                result.append(item)
        except TypeError:
            # This happens for unhashable values `item`. If this happens,
            # convert `seen` to a list and continue.
            seen = list(seen)
            seen_add = seen.append
            if item not in seen:
                seen_add(item)
                result.append(item)
    return result


def flatten_list(lst):
    result = []
    for sublist in lst:
        if not is_sequence(sublist):
            msg = ("All arguments must be lists. %s is %s")
            raise AnsibleFilterError(msg % (sublist, type(sublist)))
        if len(sublist) > 0:
            if all(is_sequence(sub) for sub in sublist):
                for item in sublist:
                    result.append(item)
            else:
                result.append(sublist)
    return result


def lists_union(*args, **kwargs):
    lists = args
    flatten = kwargs.pop('flatten', False)

    if kwargs:
        # Some unused kwargs remain
        raise AnsibleFilterError(
            "lists_union() got unexpected keywords arguments: {0}".format(
                ", ".join(kwargs.keys())
            )
        )

    if flatten:
        lists = flatten_list(args)

    if not lists:
        return []

    if len(lists) == 1:
        return lists[0]

    a = lists[0]
    for b in lists[1:]:
        a = do_union(a, b)
    return remove_duplicates(a)


def do_union(a, b):
    return a + b


def lists_intersect(*args, **kwargs):
    lists = args
    flatten = kwargs.pop('flatten', False)

    if kwargs:
        # Some unused kwargs remain
        raise AnsibleFilterError(
            "lists_intersect() got unexpected keywords arguments: {0}".format(
                ", ".join(kwargs.keys())
            )
        )

    if flatten:
        lists = flatten_list(args)

    if not lists:
        return []

    if len(lists) == 1:
        return lists[0]

    a = remove_duplicates(lists[0])
    for b in lists[1:]:
        a = do_intersect(a, b)
    return a


def do_intersect(a, b):
    isect = []
    try:
        other = set(b)
        isect = [item for item in a if item in other]
    except TypeError:
        # This happens for unhashable values,
        # use a list instead and redo.
        other = list(b)
        isect = [item for item in a if item in other]
    return isect


def lists_difference(*args, **kwargs):
    lists = args
    flatten = kwargs.pop('flatten', False)

    if kwargs:
        # Some unused kwargs remain
        raise AnsibleFilterError(
            "lists_difference() got unexpected keywords arguments: {0}".format(
                ", ".join(kwargs.keys())
            )
        )

    if flatten:
        lists = flatten_list(args)

    if not lists:
        return []

    if len(lists) == 1:
        return lists[0]

    a = remove_duplicates(lists[0])
    for b in lists[1:]:
        a = do_difference(a, b)
    return a


def do_difference(a, b):
    diff = []
    try:
        other = set(b)
        diff = [item for item in a if item not in other]
    except TypeError:
        # This happens for unhashable values,
        # use a list instead and redo.
        other = list(b)
        diff = [item for item in a if item not in other]
    return diff


def lists_symmetric_difference(*args, **kwargs):
    lists = args
    flatten = kwargs.pop('flatten', False)

    if kwargs:
        # Some unused kwargs remain
        raise AnsibleFilterError(
            "lists_difference() got unexpected keywords arguments: {0}".format(
                ", ".join(kwargs.keys())
            )
        )

    if flatten:
        lists = flatten_list(args)

    if not lists:
        return []

    if len(lists) == 1:
        return lists[0]

    a = lists[0]
    for b in lists[1:]:
        a = do_symmetric_difference(a, b)
    return a


def do_symmetric_difference(a, b):
    sym_diff = []
    union = lists_union(a, b)
    try:
        isect = set(a) & set(b)
        sym_diff = [item for item in union if item not in isect]
    except TypeError:
        # This happens for unhashable values,
        # build the intersection of `a` and `b` backed
        # by a list instead of a set and redo.
        isect = lists_intersect(a, b)
        sym_diff = [item for item in union if item not in isect]
    return sym_diff


class FilterModule(object):
    ''' Ansible lists jinja2 filters '''

    def filters(self):
        return {
            'lists_union': lists_union,
            'lists_intersect': lists_intersect,
            'lists_difference': lists_difference,
            'lists_symmetric_difference': lists_symmetric_difference,
        }



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The eos bgp_global fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from ansible.module_utils.six import iteritems
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.arista.eos.plugins.module_utils.network.eos.argspec.bgp_global.bgp_global import (
    Bgp_globalArgs,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.rm_templates.bgp_global import (
    Bgp_globalTemplate,
)


class Bgp_globalFacts(object):
    """The eos bgp_global facts class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = Bgp_globalArgs.argument_spec

    def get_config(self, connection):
        """Wrapper method for `connection.get()`
        This method exists solely to allow the unit test framework to mock device connection calls.
        """
        return connection.get("show running-config | section router\\sbgp ")

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for Bgp_global network resource

        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf

        :rtype: dictionary
        :returns: facts
        """
        facts = {}
        objs = []
        if not data:
            data = self.get_config(connection)

        # remove address_family configs from bgp_global
        bgp_global_config = []
        start = False
        self._af = False
        not_bgp = False
        for bgp_line in data.splitlines():
            if "router " in bgp_line:
                # Skip other protocol configs like router ospf etc
                if "router bgp" not in bgp_line:
                    not_bgp = True
                    continue
                not_bgp = False
            if not start and not not_bgp:
                bgp_global_config.append(bgp_line)
            if "address-family" in bgp_line:
                start = True
                self._af = True
            if start and "!" in bgp_line:
                start = False

        # parse native config using the Bgp_global template
        bgp_global_parser = Bgp_globalTemplate(
            lines=bgp_global_config,
            module=self._module,
        )
        objs = bgp_global_parser.parse()

        if objs:
            global_vals = objs.get("vrfs", {}).pop("vrf_", {})
            for key, value in iteritems(global_vals):
                objs[key] = value

            if "vrfs" in objs:
                objs["vrfs"] = list(objs["vrfs"].values())
                for vrf in objs["vrfs"]:
                    if "neighbor" in vrf:
                        vrf["neighbor"] = list(vrf["neighbor"].values())
                    if "network" in vrf:
                        vrf["network"] = list(vrf["network"].values())
                        vrf["network"] = sorted(
                            vrf["network"],
                            key=lambda k: k["address"],
                        )
                    if "aggregate_address" in vrf:
                        vrf["aggregate_address"] = sorted(
                            vrf["aggregate_address"],
                            key=lambda k: k["address"],
                        )

            if "neighbor" in objs:
                objs["neighbor"] = list(objs["neighbor"].values())

            if "network" in objs:
                objs["network"] = list(objs["network"].values())
                objs["network"] = sorted(
                    objs["network"],
                    key=lambda k: k["address"],
                )
            if "aggregate_address" in objs:
                objs["aggregate_address"] = sorted(
                    objs["aggregate_address"],
                    key=lambda k: k["address"],
                )

        ansible_facts["ansible_network_resources"].pop("bgp_global", None)

        params = utils.remove_empties(
            bgp_global_parser.validate_config(
                self.argument_spec,
                {"config": objs},
                redact=True,
            ),
        )

        facts["bgp_global"] = params.get("config", [])
        ansible_facts["ansible_network_resources"].update(facts)

        return ansible_facts



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: networks_firmware_upgrades_staged_groups
short_description: Resource module for networks _firmwareupgrades _staged _groups
description:
- Manage operations create, update and delete of the resource networks _firmwareupgrades _staged _groups.
- Create a Staged Upgrade Group for a network.
- Delete a Staged Upgrade Group.
- Update a Staged Upgrade Group for a network.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  assignedDevices:
    description: The devices and Switch Stacks assigned to the Group.
    suboptions:
      devices:
        description: Data Array of Devices containing the name and serial.
        elements: dict
        suboptions:
          name:
            description: Name of the device.
            type: str
          serial:
            description: Serial of the device.
            type: str
        type: list
      switchStacks:
        description: Data Array of Switch Stacks containing the name and id.
        elements: dict
        suboptions:
          id:
            description: ID of the Switch Stack.
            type: str
          name:
            description: Name of the Switch Stack.
            type: str
        type: list
    type: dict
  description:
    description: Description of the Staged Upgrade Group. Length must be 1 to 255 characters.
    type: str
  groupId:
    description: GroupId path parameter. Group ID.
    type: str
  isDefault:
    description: Boolean indicating the default Group. Any device that does not have
      a group explicitly assigned will upgrade with this group.
    type: bool
  name:
    description: Name of the Staged Upgrade Group. Length must be 1 to 255 characters.
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for networks createNetworkFirmwareUpgradesStagedGroup
  description: Complete reference of the createNetworkFirmwareUpgradesStagedGroup API.
  link: https://developer.cisco.com/meraki/api-v1/#!create-network-firmware-upgrades-staged-group
- name: Cisco Meraki documentation for networks deleteNetworkFirmwareUpgradesStagedGroup
  description: Complete reference of the deleteNetworkFirmwareUpgradesStagedGroup API.
  link: https://developer.cisco.com/meraki/api-v1/#!delete-network-firmware-upgrades-staged-group
- name: Cisco Meraki documentation for networks updateNetworkFirmwareUpgradesStagedGroup
  description: Complete reference of the updateNetworkFirmwareUpgradesStagedGroup API.
  link: https://developer.cisco.com/meraki/api-v1/#!update-network-firmware-upgrades-staged-group
notes:
  - SDK Method used are
    networks.Networks.create_network_firmware_upgrades_staged_group,
    networks.Networks.delete_network_firmware_upgrades_staged_group,
    networks.Networks.update_network_firmware_upgrades_staged_group,

  - Paths used are
    post /networks/{networkId}/firmwareUpgrades/staged/groups,
    delete /networks/{networkId}/firmwareUpgrades/staged/groups/{groupId},
    put /networks/{networkId}/firmwareUpgrades/staged/groups/{groupId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_firmware_upgrades_staged_groups:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    assignedDevices:
      devices:
      - name: Device Name
        serial: Q234-ABCD-5678
      switchStacks:
      - id: '1234'
        name: Stack Name
    description: The description of the group
    isDefault: false
    name: My Staged Upgrade Group
    networkId: string

- name: Update by id
  cisco.meraki.networks_firmware_upgrades_staged_groups:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    assignedDevices:
      devices:
      - name: Device Name
        serial: Q234-ABCD-5678
      switchStacks:
      - id: '1234'
        name: Stack Name
    description: The description of the group
    groupId: string
    isDefault: false
    name: My Staged Upgrade Group
    networkId: string

- name: Delete by id
  cisco.meraki.networks_firmware_upgrades_staged_groups:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: absent
    groupId: string
    networkId: string

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



#!/usr/bin/python
#
# Copyright (c) 2019 Zim Kalinowski, (@zikalino)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_hdinsightcluster_info
version_added: "0.1.2"
short_description: Get Azure HDInsight Cluster facts
description:
    - Get facts of Azure HDInsight Cluster.

options:
    resource_group:
        description:
            - Name of an Azure resource group.
        type: str
    name:
        description:
            - HDInsight cluster name.
        type: str
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.
        type: list
        elements: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Zim Kalinowski (@zikalino)

'''

EXAMPLES = '''
- name: Get instance of HDInsight Cluster
  azure_rm_hdinsightcluster_info:
    resource_group: myResourceGroup
    name: myCluster

- name: List instances of HDInsight Cluster
  azure_rm_hdinsightcluster_info:
    resource_group: myResourceGroup
    tags:
      - key:value
'''

RETURN = '''
clusters:
    description:
        - A list of dictionaries containing facts for HDInsight Cluster.
    returned: always
    type: complex
    contains:
        id:
            description:
                - The unique resource identifier of the HDInsight Cluster.
            returned: always
            type: str
            sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.HDInsight/clusters/myCluster"
        resource_group:
            description:
                - Name of an Azure resource group.
            returned: always
            type: str
            sample: myResourceGroup
        name:
            description:
                - The name of the HDInsight Cluster.
            returned: always
            type: str
            sample: testaccount
        location:
            description:
                - The location of the resource group to which the resource belongs.
            returned: always
            type: str
            sample: westus
        cluster_version:
            description:
                - The version of the cluster.
            returned: always
            type: str
            sample: 3.6.1000.67
        os_type:
            description:
                - The type of operating system.
            returned: always
            type: str
            sample: linux
        tier:
            description:
                - The cluster tier.
            returned: always
            type: str
            sample: standard
        cluster_definition:
            description:
                - The cluster definition.
            type: complex
            contains:
                kind:
                    description:
                        - The type of cluster.
                    returned: always
                    type: str
                    sample: spark
        compute_profile_roles:
            description:
                - The list of roles in the cluster.
            type: complex
            contains:
                name:
                    description:
                        - The name of the role.
                    returned: always
                    type: str
                    sample: headnode
                target_instance_count:
                    description:
                        - The instance count of the cluster.
                    returned: always
                    type: int
                    sample: 2
                vm_size:
                    description:
                        - The size of the VM.
                    returned: always
                    type: str
                    sample: Standard_D3
                linux_profile:
                    description:
                        - The Linux OS profile.
                    type: dict
                    contains:
                        username:
                            description:
                                - User name.
                            returned: always
                            type: str
                            sample: myuser
        connectivity_endpoints:
            description:
                - Cluster's connectivity endpoints.
            type: complex
            contains:
                location:
                    description:
                        - Endpoint location.
                    returned: always
                    type: str
                    sample: myCluster-ssh.azurehdinsight.net
                name:
                    description:
                        - Endpoint name.
                    returned: always
                    type: str
                    sample: SSH
                port:
                    description:
                        - Endpoint port.
                    returned: always
                    type: int
                    sample: 22
                protocol:
                    description:
                        - Endpoint protocol.
                    returned: always
                    type: str
                    sample: TCP
        tags:
            description:
                - The tags of the resource.
            returned: always
            type: dict
            sample: {}
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError
    from azure.mgmt.hdinsight import HDInsightManagementClient
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMHDInsightclusterInfo(AzureRMModuleBase):
    def __init__(self):
        # define user inputs into argument
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str'
            ),
            name=dict(
                type='str'
            ),
            tags=dict(
                type='list',
                elements='str'
            )
        )
        # store the results of the module operation
        self.results = dict(
            changed=False
        )
        self.mgmt_client = None
        self.resource_group = None
        self.name = None
        self.tags = None

        super(AzureRMHDInsightclusterInfo, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=False, facts_module=True)

    def exec_module(self, **kwargs):

        is_old_facts = self.module._name == 'azure_rm_hdinsightcluster_facts'
        if is_old_facts:
            self.module.deprecate("The 'azure_rm_hdinsightcluster_facts' module has been renamed to 'azure_rm_hdinsightcluster_info'",
                                  version=(2.9, ))

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])
        self.mgmt_client = self.get_mgmt_svc_client(HDInsightManagementClient,
                                                    is_track2=True,
                                                    base_url=self._cloud_environment.endpoints.resource_manager)

        if self.name is not None:
            self.results['clusters'] = self.get()
        elif self.resource_group is not None:
            self.results['clusters'] = self.list_by_resource_group()
        else:
            self.results['clusters'] = self.list_all()
        return self.results

    def get(self):
        response = None
        results = []
        try:
            response = self.mgmt_client.clusters.get(resource_group_name=self.resource_group,
                                                     cluster_name=self.name)
            self.log("Response : {0}".format(response))
        except ResourceNotFoundError as e:
            self.log('Could not get facts for HDInsight Cluster.')

        if response and self.has_tags(response.tags, self.tags):
            results.append(self.format_response(response))

        return results

    def list_by_resource_group(self):
        response = None
        results = []
        try:
            response = self.mgmt_client.clusters.list_by_resource_group(resource_group_name=self.resource_group)
            self.log("Response : {0}".format(response))
        except Exception as e:
            self.log('Could not get facts for HDInsight Cluster.')

        if response is not None:
            for item in response:
                if self.has_tags(item.tags, self.tags):
                    results.append(self.format_response(item))

        return results

    def list_all(self):
        response = None
        results = []
        try:
            response = self.mgmt_client.clusters.list()
            self.log("Response : {0}".format(response))
        except Exception as e:
            self.log('Could not get facts for HDInsight Cluster.')

        if response is not None:
            for item in response:
                if self.has_tags(item.tags, self.tags):
                    results.append(self.format_response(item))

        return results

    def format_response(self, item):
        d = item.as_dict()
        d = {
            'id': d.get('id'),
            'resource_group': self.parse_resource_to_dict(d.get('id')).get('resource_group'),
            'name': d.get('name', None),
            'location': d.get('location', '').replace(' ', '').lower(),

            'cluster_version': d.get('properties', {}).get('cluster_version'),
            'os_type': d.get('properties', {}).get('os_type'),
            'tier': d.get('properties', {}).get('tier'),
            'cluster_definition': {
                'kind': d.get('properties', {}).get('cluster_definition', {}).get('kind')
            },
            'compute_profile_roles': [{
                'name': item.get('name'),
                'target_instance_count': item.get('target_instance_count'),
                'vm_size': item.get('hardware_profile', {}).get('vm_size'),
                'linux_profile': {
                    'username': item.get('os_profile', {}).get('linux_operating_system_profile', {}).get('username')
                }
            } for item in d.get('properties', []).get('compute_profile', {}).get('roles', [])],
            'connectivity_endpoints': d.get('properties', {}).get('connectivity_endpoints'),
            'tags': d.get('tags', None)
        }

        return d


def main():
    AzureRMHDInsightclusterInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: nxos_snmp_user
extends_documentation_fragment:
- cisco.nxos.nxos
short_description: (deprecated, removed after 2024-01-01) Manages SNMP users for monitoring.
description:
- Manages SNMP user configuration.
version_added: 1.0.0
deprecated:
  alternative: nxos_snmp_server
  why: Updated modules released with more functionality
  removed_at_date: '2024-01-01'
author:
- Jason Edelman (@jedelman8)
notes:
- Tested against NXOSv 7.3.(0)D1(1) on VIRL
- Limited Support for Cisco MDS
- Authentication parameters not idempotent.
options:
  user:
    description:
    - Name of the user.
    required: true
    type: str
  group:
    description:
    - Group to which the user will belong to. If state = present, and the user is
      existing, the group is added to the user. If the user is not existing, user
      entry is created with this group argument. If state = absent, only the group
      is removed from the user entry. However, to maintain backward compatibility,
      if the existing user belongs to only one group, and if group argument is same
      as the existing user's group, then the user entry also is deleted.
    type: str
  authentication:
    description:
    - Authentication parameters for the user.
    choices:
    - md5
    - sha
    type: str
  pwd:
    description:
    - Authentication password when using md5 or sha. This is not idempotent
    type: str
  privacy:
    description:
    - Privacy password for the user. This is not idempotent
    type: str
  encrypt:
    description:
    - Enables AES-128 bit encryption when using privacy password.
    type: bool
  state:
    description:
    - Manage the state of the resource.
    default: present
    choices:
    - present
    - absent
    type: str
"""

EXAMPLES = """
- cisco.nxos.nxos_snmp_user:
    user: ntc
    group: network-operator
    authentication: md5
    pwd: test_password
"""

RETURN = """
commands:
    description: commands sent to the device
    returned: always
    type: list
    sample: ["snmp-server user ntc network-operator auth md5 test_password"]
"""

import re

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    load_config,
    run_commands,
)


def execute_show_command(command, module, text=False):
    command = {"command": command, "output": "json"}
    if text:
        command["output"] = "text"

    return run_commands(module, command)


def flatten_list(command_lists):
    flat_command_list = []
    for command in command_lists:
        if isinstance(command, list):
            flat_command_list.extend(command)
        else:
            flat_command_list.append(command)
    return flat_command_list


def get_snmp_groups(module):
    data = execute_show_command("show snmp group", module)[0]
    group_list = []

    try:
        group_table = data["TABLE_role"]["ROW_role"]
        for group in group_table:
            group_list.append(group["role_name"])
    except (KeyError, AttributeError):
        return group_list

    return group_list


def get_snmp_user(user, module):
    command = "show snmp user {0}".format(user)
    body = execute_show_command(command, module, text=True)
    body_text = body[0]

    if "No such entry" not in body[0]:
        body = execute_show_command(command, module)

    resource = {}
    try:
        # The TABLE and ROW keys differ between NXOS platforms.
        if body[0].get("TABLE_snmp_user"):
            tablekey = "TABLE_snmp_user"
            rowkey = "ROW_snmp_user"
            tablegrpkey = "TABLE_snmp_group_names"
            rowgrpkey = "ROW_snmp_group_names"
            authkey = "auth_protocol"
            privkey = "priv_protocol"
            grpkey = "group_names"
        elif body[0].get("TABLE_snmp_users"):
            tablekey = "TABLE_snmp_users"
            rowkey = "ROW_snmp_users"
            tablegrpkey = "TABLE_groups"
            rowgrpkey = "ROW_groups"
            authkey = "auth"
            privkey = "priv"
            grpkey = "group"

        rt = body[0][tablekey][rowkey]
        # on some older platforms, all groups except the 1st one
        # are in list elements by themselves and they are
        # indexed by 'user'. This is due to a platform bug.
        # Get first element if rt is a list due to the bug
        # or if there is no bug, parse rt directly
        if isinstance(rt, list):
            resource_table = rt[0]
        else:
            resource_table = rt

        resource["user"] = user
        resource["authentication"] = str(resource_table[authkey]).strip()
        encrypt = str(resource_table[privkey]).strip()
        if encrypt.startswith("aes"):
            resource["encrypt"] = "aes-128"
        else:
            resource["encrypt"] = "none"

        groups = []
        if tablegrpkey in resource_table:
            group_table = resource_table[tablegrpkey][rowgrpkey]
            try:
                for group in group_table:
                    groups.append(str(group[grpkey]).strip())
            except TypeError:
                groups.append(str(group_table[grpkey]).strip())

            # Now for the platform bug case, get the groups
            if isinstance(rt, list):
                # remove 1st element from the list as this is parsed already
                rt.pop(0)
                # iterate through other elements indexed by
                # 'user' and add it to groups.
                for each in rt:
                    groups.append(each["user"].strip())

        # Some 'F' platforms use 'group' key instead
        elif "group" in resource_table:
            # single group is a string, multiple groups in a list
            groups = resource_table["group"]
            if isinstance(groups, str):
                groups = [groups]

        resource["group"] = groups

    except (KeyError, AttributeError, IndexError, TypeError):
        if not resource and body_text and "No such entry" not in body_text:
            # 6K and other platforms may not return structured output;
            # attempt to get state from text output
            resource = get_non_structured_snmp_user(body_text)

    return resource


def get_non_structured_snmp_user(body_text):
    # This method is a workaround for platforms that don't support structured
    # output for 'show snmp user <foo>'. This workaround may not work on all
    # platforms. Sample non-struct output:
    #
    # User                Auth  Priv(enforce) Groups              acl_filter
    # ____                ____  _____________ ______              __________
    # sample1             no    no            network-admin       ipv4:my_acl
    #                                         network-operator
    #                                         priv-11
    #         -OR-
    # sample2             md5   des(no)       priv-15
    #         -OR-
    # sample3             md5   aes-128(no)   network-admin
    resource = {}
    output = body_text.rsplit("__________")[-1]
    pat = re.compile(
        r"^(?P<user>\S+)\s+"
        r"(?P<auth>\S+)\s+"
        r"(?P<priv>[\w\d-]+)(?P<enforce>\([\w\d-]+\))*\s+"
        r"(?P<group>\S+)",
        re.M,
    )
    m = re.search(pat, output)
    if not m:
        return resource
    resource["user"] = m.group("user")
    resource["auth"] = m.group("auth")
    resource["encrypt"] = "aes-128" if "aes" in str(m.group("priv")) else "none"

    resource["group"] = [m.group("group")]
    more_groups = re.findall(r"^\s+([\w\d-]+)\s*$", output, re.M)
    if more_groups:
        resource["group"] += more_groups

    return resource


def remove_snmp_user(user, group=None):
    if group:
        return ["no snmp-server user {0} {1}".format(user, group)]
    else:
        return ["no snmp-server user {0}".format(user)]


def config_snmp_user(proposed, user, reset):
    if reset:
        commands = remove_snmp_user(user)
    else:
        commands = []

    if proposed.get("group"):
        cmd = "snmp-server user {0} {group}".format(user, **proposed)
    else:
        cmd = "snmp-server user {0}".format(user)

    auth = proposed.get("authentication", None)
    pwd = proposed.get("pwd", None)

    if auth and pwd:
        cmd += " auth {authentication} {pwd}".format(**proposed)

    encrypt = proposed.get("encrypt", None)
    privacy = proposed.get("privacy", None)

    if encrypt and privacy:
        cmd += " priv {encrypt} {privacy}".format(**proposed)
    elif privacy:
        cmd += " priv {privacy}".format(**proposed)

    if cmd:
        commands.append(cmd)

    return commands


def main():
    argument_spec = dict(
        user=dict(required=True, type="str"),
        group=dict(type="str"),
        pwd=dict(type="str", no_log=True),
        privacy=dict(type="str"),
        authentication=dict(choices=["md5", "sha"]),
        encrypt=dict(type="bool"),
        state=dict(choices=["absent", "present"], default="present"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        required_together=[["authentication", "pwd"], ["encrypt", "privacy"]],
        supports_check_mode=True,
    )

    warnings = list()
    results = {"changed": False, "commands": [], "warnings": warnings}

    user = module.params["user"]
    group = module.params["group"]
    pwd = module.params["pwd"]
    privacy = module.params["privacy"]
    encrypt = module.params["encrypt"]
    authentication = module.params["authentication"]
    state = module.params["state"]

    if privacy and encrypt:
        if not pwd and authentication:
            module.fail_json(
                msg="pwd and authentication must be provided " "when using privacy and encrypt",
            )

    if group and group not in get_snmp_groups(module):
        module.fail_json(msg="group not configured yet on switch.")

    existing = get_snmp_user(user, module)

    if state == "present" and existing:
        if group:
            if group not in existing["group"]:
                existing["group"] = None
            else:
                existing["group"] = group
        else:
            existing["group"] = None

    commands = []

    if state == "absent" and existing:
        if group:
            if group in existing["group"]:
                if len(existing["group"]) == 1:
                    commands.append(remove_snmp_user(user))
                else:
                    commands.append(remove_snmp_user(user, group))
        else:
            commands.append(remove_snmp_user(user))

    elif state == "present":
        reset = False

        args = dict(
            user=user,
            pwd=pwd,
            group=group,
            privacy=privacy,
            encrypt=encrypt,
            authentication=authentication,
        )
        proposed = dict((k, v) for k, v in args.items() if v is not None)

        if not existing:
            if encrypt:
                proposed["encrypt"] = "aes-128"
            commands.append(config_snmp_user(proposed, user, reset))

        elif existing:
            if encrypt and not existing["encrypt"].startswith("aes"):
                reset = True
                proposed["encrypt"] = "aes-128"

            delta = dict(set(proposed.items()).difference(existing.items()))

            if delta.get("pwd"):
                delta["authentication"] = authentication

            if delta and encrypt:
                delta["encrypt"] = "aes-128"

            if delta:
                command = config_snmp_user(delta, user, reset)
                commands.append(command)

    cmds = flatten_list(commands)
    if cmds:
        results["changed"] = True
        if not module.check_mode:
            load_config(module, cmds)

        if "configure" in cmds:
            cmds.pop(0)
        results["commands"] = cmds

    module.exit_json(**results)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

# Copyright: (c) 2017, Dag Wieers (@dagwieers) <dag@wieers.com>
# Copyright: (c) 2017, Swetha Chunduri (@schunduri)
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


class ModuleDocFragment(object):
    # Standard files documentation fragment
    DOCUMENTATION = r"""
options:
  host:
    description:
    - IP Address or hostname of APIC resolvable by Ansible control host.
    - If the value is not specified in the task, the value of environment variable C(ACI_HOST) will be used instead.
    type: str
    aliases: [ hostname ]
  port:
    description:
    - Port number to be used for REST connection.
    - The default value depends on parameter C(use_ssl).
    - If the value is not specified in the task, the value of environment variable C(ACI_PORT) will be used instead.
    type: int
  username:
    description:
    - The username to use for authentication.
    - If the value is not specified in the task, the value of environment variables C(ACI_USERNAME) or C(ANSIBLE_NET_USERNAME) will be used instead.
    - The default value is admin.
    type: str
    aliases: [ user ]
  password:
    description:
    - The password to use for authentication.
    - This option is mutual exclusive with C(private_key). If C(private_key) is provided too, it will be used instead.
    - If the value is not specified in the task, the value of environment variables C(ACI_PASSWORD) or C(ANSIBLE_NET_PASSWORD) will be used instead.
    type: str
  private_key:
    description:
    - Either a PEM-formatted private key file or the private key content used for signature-based authentication.
    - This value also influences the default C(certificate_name) that is used.
    - This option is mutual exclusive with C(password). If C(password) is provided too, it will be ignored.
    - If the value is not specified in the task, the value of environment variable C(ACI_PRIVATE_KEY) or C(ANSIBLE_NET_SSH_KEYFILE) will be used instead.
    type: str
    aliases: [ cert_key ]
  certificate_name:
    description:
    - The X.509 certificate name attached to the APIC AAA user used for signature-based authentication.
    - If a C(private_key) filename was provided, this defaults to the C(private_key) basename, without extension.
    - If PEM-formatted content was provided for C(private_key), this defaults to the C(username) value.
    - If the value is not specified in the task, the value of environment variable C(ACI_CERTIFICATE_NAME) will be used instead.
    type: str
    aliases: [ cert_name ]
  output_level:
    description:
    - Influence the output of this ACI module.
    - C(normal) means the standard output, incl. C(current) dict
    - C(info) adds informational output, incl. C(previous), C(proposed) and C(sent) dicts
    - C(debug) adds debugging output, incl. C(filter_string), C(method), C(response), C(status) and C(url) information
    - If the value is not specified in the task, the value of environment variable C(ACI_OUTPUT_LEVEL) will be used instead.
    type: str
    choices: [ debug, info, normal ]
    default: normal
  timeout:
    description:
    - The socket level timeout in seconds.
    - If the value is not specified in the task, the value of environment variable C(ACI_TIMEOUT) will be used instead.
    - The default value is 30.
    type: int
  use_proxy:
    description:
    - If C(false), it will not use a proxy, even if one is defined in an environment variable on the target hosts.
    - If the value is not specified in the task, the value of environment variable C(ACI_USE_PROXY) will be used instead.
    - The default value is true.
    type: bool
  use_ssl:
    description:
    - If C(false), an HTTP connection will be used instead of the default HTTPS connection.
    - If the value is not specified in the task, the value of environment variable C(ACI_USE_SSL) will be used instead.
    - The default value is true when the connection is local.
    type: bool
  validate_certs:
    description:
    - If C(false), SSL certificates will not be validated.
    - This should only set to C(false) when used on personally controlled sites using self-signed certificates.
    - If the value is not specified in the task, the value of environment variable C(ACI_VALIDATE_CERTS) will be used instead.
    - The default value is true.
    type: bool
  output_path:
    description:
    - Path to a file that will be used to dump the ACI JSON configuration objects generated by the module.
    - If the value is not specified in the task, the value of environment variable C(ACI_OUTPUT_PATH) will be used instead.
    type: str
seealso:
- ref: aci_guide
  description: Detailed information on how to manage your ACI infrastructure using Ansible.
- ref: aci_dev_guide
  description: Detailed guide on how to write your own Cisco ACI modules to contribute.
"""



# -*- coding: utf-8 -*-
# Copyright 2022 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Hostname parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


class HostnameTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(HostnameTemplate, self).__init__(lines=lines, tmplt=self, module=module)

    # fmt: off
    PARSERS = [
        {
            "name": "hostname",
            "getval": re.compile(
                r"""
                ^hostname\s(?P<hostname>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "hostname {{ hostname }}",
            "result": {
                "hostname": "{{ hostname }}",
            },
        },
    ]
    # fmt: on



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage Check Point Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: cp_mgmt_assign_global_assignment
short_description: assign global assignment on Check Point over Web Services API
description:
  - assign global assignment on Check Point over Web Services API
  - All operations are performed over Web Services API.
version_added: "1.0.0"
author: "Or Soffer (@chkp-orso)"
options:
  dependent_domains:
    description:
      - N/A
    type: list
    elements: str
  global_domains:
    description:
      - N/A
    type: list
    elements: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
extends_documentation_fragment: check_point.mgmt.checkpoint_commands
"""

EXAMPLES = """
- name: assign-global-assignment
  cp_mgmt_assign_global_assignment:
    dependent_domains: domain1
    global_domains: Global2
"""

RETURN = """
cp_mgmt_assign_global_assignment:
  description: The checkpoint assign-global-assignment output.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import (
    checkpoint_argument_spec_for_commands,
    api_command,
)


def main():
    argument_spec = dict(
        dependent_domains=dict(type="list", elements="str"),
        global_domains=dict(type="list", elements="str"),
        details_level=dict(type="str", choices=["uid", "standard", "full"]),
    )
    argument_spec.update(checkpoint_argument_spec_for_commands)

    module = AnsibleModule(argument_spec=argument_spec)

    command = "assign-global-assignment"

    result = api_command(module, command)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright: (c) 2021, Ansible Project
# Copyright: (c) 2021, Mark Mercado <mamercad@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


DOCUMENTATION = r"""
---
module: digital_ocean_balance_info
short_description: Display DigitalOcean customer balance
description:
  - This module can be used to display the DigitalOcean customer balance.
author: "Mark Mercado (@mamercad)"
version_added: 1.2.0
extends_documentation_fragment:
  - community.digitalocean.digital_ocean.documentation
"""


EXAMPLES = r"""
- name: Display DigitalOcean customer balance
  community.digitalocean.digital_ocean_balance_info:
    oauth_token: "{{ oauth_token }}"
"""


RETURN = r"""
# DigitalOcean API info https://docs.digitalocean.com/reference/api/api-reference/#operation/get_customer_balance
data:
    description: DigitalOcean customer balance
    returned: success
    type: dict
    sample: {
        "account_balance": "-27.52",
        "generated_at": "2021-04-11T05:08:24Z",
        "month_to_date_balance": "-27.40",
        "month_to_date_usage": "0.00"
    }
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.digitalocean.plugins.module_utils.digital_ocean import (
    DigitalOceanHelper,
)


def run(module):
    rest = DigitalOceanHelper(module)

    response = rest.get("customers/my/balance")
    if response.status_code != 200:
        module.fail_json(
            msg="Failed to fetch 'customers/my/balance' information due to error : %s"
            % response.json["message"]
        )

    module.exit_json(changed=False, data=response.json)


def main():
    argument_spec = DigitalOceanHelper.digital_ocean_argument_spec()
    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    run(module)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: organizations_devices_provisioning_statuses_info
short_description: Information module for organizations _devices _provisioning _statuses
description:
- Get all organizations _devices _provisioning _statuses.
- List the provisioning statuses information for devices in an organization.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
    - OrganizationId path parameter. Organization ID.
    type: str
  perPage:
    description:
    - PerPage query parameter. The number of entries per page returned. Acceptable range is 3 - 1000. Default is 1000.
    type: int
  startingAfter:
    description:
    - >
      StartingAfter query parameter. A token used by the server to indicate the start of the page. Often this is a
      timestamp or an ID but it is not limited to those. This parameter should not be defined by client
      applications. The link for the first, last, prev, or next page in the HTTP Link header should define it.
    type: str
  endingBefore:
    description:
    - >
      EndingBefore query parameter. A token used by the server to indicate the end of the page. Often this is a
      timestamp or an ID but it is not limited to those. This parameter should not be defined by client
      applications. The link for the first, last, prev, or next page in the HTTP Link header should define it.
    type: str
  networkIds:
    description:
    - >
      NetworkIds query parameter. Optional parameter to filter device by network ID. This filter uses multiple
      exact matches.
    elements: str
    type: list
  productTypes:
    description:
    - >
      ProductTypes query parameter. Optional parameter to filter device by device product types. This filter uses
      multiple exact matches.
    elements: str
    type: list
  serials:
    description:
    - >
      Serials query parameter. Optional parameter to filter device by device serial numbers. This filter uses
      multiple exact matches.
    elements: str
    type: list
  status:
    description:
    - >
      Status query parameter. An optional parameter to filter devices by the provisioning status. Accepted
      statuses unprovisioned, incomplete, complete.
    type: str
  tags:
    description:
    - >
      Tags query parameter. An optional parameter to filter devices by tags. The filtering is case-sensitive. If
      tags are included, 'tagsFilterType' should also be included (see below). This filter uses multiple exact
      matches.
    elements: str
    type: list
  tagsFilterType:
    description:
    - >
      TagsFilterType query parameter. An optional parameter of value 'withAnyTags' or 'withAllTags' to indicate
      whether to return devices which contain ANY or ALL of the included tags. If no type is included,
      'withAnyTags' will be selected.
    type: str
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for organizations getOrganizationDevicesProvisioningStatuses
  description: Complete reference of the getOrganizationDevicesProvisioningStatuses API.
  link: https://developer.cisco.com/meraki/api-v1/#!get-organization-devices-provisioning-statuses
notes:
  - SDK Method used are
    organizations.Organizations.get_organization_devices_provisioning_statuses,

  - Paths used are
    get /organizations/{organizationId}/devices/provisioning/statuses,
"""

EXAMPLES = r"""
- name: Get all organizations _devices _provisioning _statuses
  cisco.meraki.organizations_devices_provisioning_statuses_info:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    perPage: 0
    startingAfter: string
    endingBefore: string
    networkIds: []
    productTypes: []
    serials: []
    status: string
    tags: []
    tagsFilterType: string
    organizationId: string
    total_pages: -1
    direction: next
  register: result

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "mac": "string",
        "name": "string",
        "network": {
          "id": "string"
        },
        "productType": "string",
        "serial": "string",
        "status": "string",
        "tags": [
          "string"
        ]
      }
    ]
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: networks_cellular_gateway_dhcp
short_description: Resource module for networks _cellulargateway _dhcp
description:
- Manage operation update of the resource networks _cellulargateway _dhcp.
- Update common DHCP settings of MGs.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  dhcpLeaseTime:
    description: DHCP Lease time for all MG of the network. Possible values are '30
      minutes', '1 hour', '4 hours', '12 hours', '1 day' or '1 week'.
    type: str
  dnsCustomNameservers:
    description: List of fixed IPs representing the the DNS Name servers when the mode
      is 'custom'.
    elements: str
    type: list
  dnsNameservers:
    description: DNS name servers mode for all MG of the network. Possible values are
      'upstream_dns', 'google_dns', 'opendns', 'custom'.
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for cellularGateway updateNetworkCellularGatewayDhcp
  description: Complete reference of the updateNetworkCellularGatewayDhcp API.
  link: https://developer.cisco.com/meraki/api-v1/#!update-network-cellular-gateway-dhcp
notes:
  - SDK Method used are
    cellular_gateway.CellularGateway.update_network_cellular_gateway_dhcp,

  - Paths used are
    put /networks/{networkId}/cellularGateway/dhcp,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.networks_cellular_gateway_dhcp:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    dhcpLeaseTime: 1 hour
    dnsCustomNameservers:
    - 172.16.2.111
    - 172.16.2.30
    dnsNameservers: custom
    networkId: string

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "dhcpLeaseTime": "string",
      "dnsNameservers": "string",
      "dnsCustomNameservers": [
        "string"
      ]
    }
"""



# -*- coding: utf-8 -*-

# Copyright (c) 2016 Michael Gruener <michael.gruener@chaosmoon.net>
# Copyright (c) 2021 Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function
__metaclass__ = type


import abc

from ansible.module_utils import six


@six.add_metaclass(abc.ABCMeta)
class CryptoBackend(object):
    def __init__(self, module):
        self.module = module

    @abc.abstractmethod
    def parse_key(self, key_file=None, key_content=None, passphrase=None):
        '''
        Parses an RSA or Elliptic Curve key file in PEM format and returns key_data.
        Raises KeyParsingError in case of errors.
        '''

    @abc.abstractmethod
    def sign(self, payload64, protected64, key_data):
        pass

    @abc.abstractmethod
    def create_mac_key(self, alg, key):
        '''Create a MAC key.'''

    @abc.abstractmethod
    def get_csr_identifiers(self, csr_filename=None, csr_content=None):
        '''
        Return a set of requested identifiers (CN and SANs) for the CSR.
        Each identifier is a pair (type, identifier), where type is either
        'dns' or 'ip'.
        '''

    @abc.abstractmethod
    def get_cert_days(self, cert_filename=None, cert_content=None, now=None):
        '''
        Return the days the certificate in cert_filename remains valid and -1
        if the file was not found. If cert_filename contains more than one
        certificate, only the first one will be considered.

        If now is not specified, datetime.datetime.now() is used.
        '''

    @abc.abstractmethod
    def create_chain_matcher(self, criterium):
        '''
        Given a Criterium object, creates a ChainMatcher object.
        '''



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: cp_mgmt_add_rules_batch
short_description: Creates new rules in batch. Use this API to achieve optimum performance when adding more than one rule.
description:
  - Creates new rules in batch. Use this API to achieve optimum performance when adding more than one rule.
  - Add multiple rules to a layer in a specific position, incrementing position by one for each rule.
  - Errors and warnings are ignored when using this API, operation will apply changes while ignoring errors. It is not
    possible to publish changes that contain validations errors. You must use the "show-validations" API to see any
    validation errors and warnings caused by the batch creation. Supported rules types are access-rule, nat-rule,
    https-rule and threat-exception.
  - This module is not idempotent.
  - All operations are performed over Web Services API.
version_added: "3.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  objects:
    description:
      - Batch of rules separated by types.
    type: list
    elements: dict
    suboptions:
      layer:
        description:
          - Layer name or uid.
        type: str
      type:
        description:
          - Type of rules to be created. <br>Only types from above are supported.
        type: str
      first_position:
        description:
          - First rule position.
        type: str
      list:
        description:
          - List of rules from the same type to be created on the same layer. <br>Use the "add" API reference documentation for a single rule
            command to find the expected fields for the request. <br>For example, to add access-rules, use the "add-access-rule" command found in the API
            reference documentation (under Access Control & NAT). <br>Note, "set-if-exists", "ignore-errors", "ignore-warnings" and "details-level" options
            are not supported when adding a batch of rules.
        type: list
        elements: dict
  auto_publish_session:
    description:
    - Publish the current session if changes have been performed after task completes.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_commands
"""

EXAMPLES = """
- name: add-rules-batch
  cp_mgmt_add_rules_batch:
    objects:
    - first_position: top
      layer: Network
      list:
      - action: accept
        name: access rule 1
      - action: accept
        name: access rule 2
      type: access-rule
    - first_position: top
      layer: Standard
      list:
      - name: nat rule 1
      - name: nat rule 2
      type: nat-rule
    - first_position: top
      layer: Default Layer
      list:
      - name: https rule 1
      - name: https rule 2
      type: https-rule

"""

RETURN = """
cp_mgmt_add_rules_batch:
  description: The checkpoint add-rules-batch output.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import (
    checkpoint_argument_spec_for_commands,
    api_command,
)


def main():
    argument_spec = dict(
        objects=dict(
            type="list",
            elements="dict",
            options=dict(
                layer=dict(type="str"),
                type=dict(type="str"),
                first_position=dict(type="str"),
                list=dict(type="list", elements="dict"),
            ),
        ),
        auto_publish_session=dict(type="bool"),
    )
    argument_spec.update(checkpoint_argument_spec_for_commands)

    module = AnsibleModule(argument_spec=argument_spec)

    command = "add-rules-batch"

    result = api_command(module, command)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2017, Gaudenz Steinlin <gaudenz.steinlin@cloudscale.ch>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: floating_ip
short_description: Manages floating IPs on the cloudscale.ch IaaS service
description:
  - Create, assign and delete floating IPs on the cloudscale.ch IaaS service.
notes:
  - Once a floating_ip is created, all parameters except C(server), C(reverse_ptr) and C(tags) are read-only.
author:
  - Gaudenz Steinlin (@gaudenz)
  - Denis Krienbühl (@href)
  - René Moser (@resmo)
version_added: 1.0.0
options:
  network:
    description:
      - Floating IP address to change.
      - One of I(network) or I(name) is required to identify the floating IP.
    aliases: [ ip ]
    type: str
  name:
    description:
      - Name to identifiy the floating IP address for idempotency.
      - One of I(network) or I(name) is required to identify the floating IP.
      - Required for assigning a new floating IP.
    version_added: 1.3.0
    type: str
  state:
    description:
      - State of the floating IP.
    default: present
    choices: [ present, absent ]
    type: str
  ip_version:
    description:
      - IP protocol version of the floating IP.
      - Required when assigning a new floating IP.
    choices: [ 4, 6 ]
    type: int
  server:
    description:
      - UUID of the server assigned to this floating IP.
    type: str
  type:
    description:
      - The type of the floating IP.
    choices: [ regional, global ]
    type: str
    default: regional
  region:
    description:
      - Region in which the floating IP resides (e.g. C(lpg) or C(rma)).
        If omitted, the region of the project default zone is used.
        This parameter must be omitted if I(type) is set to C(global).
    type: str
  prefix_length:
    description:
      - Only valid if I(ip_version) is 6.
      - Prefix length for the IPv6 network. Currently only a prefix of /56 can be requested. If no I(prefix_length) is present, a
        single address is created.
    choices: [ 56 ]
    type: int
  reverse_ptr:
    description:
      - Reverse PTR entry for this address.
      - You cannot set a reverse PTR entry for IPv6 floating networks. Reverse PTR entries are only allowed for single addresses.
    type: str
  tags:
    description:
      - Tags associated with the floating IP. Set this to C({}) to clear any tags.
    type: dict
    version_added: 1.1.0
extends_documentation_fragment: cloudscale_ch.cloud.api_parameters
'''

EXAMPLES = '''
# Request a new floating IP without assignment to a server
- name: Request a floating IP
  cloudscale_ch.cloud.floating_ip:
    name: IP to my server
    ip_version: 4
    reverse_ptr: my-server.example.com
    api_token: xxxxxx

# Request a new floating IP with assignment
- name: Request a floating IP
  cloudscale_ch.cloud.floating_ip:
    name: web
    ip_version: 4
    server: 47cec963-fcd2-482f-bdb6-24461b2d47b1
    reverse_ptr: my-server.example.com
    api_token: xxxxxx

# Assign an existing floating IP to a different server by its IP address
- name: Move floating IP to backup server
  cloudscale_ch.cloud.floating_ip:
    ip: 192.0.2.123
    server: ea3b39a3-77a8-4d0b-881d-0bb00a1e7f48
    api_token: xxxxxx

# Assign an existing floating IP to a different server by name
- name: Move floating IP to backup server
  cloudscale_ch.cloud.floating_ip:
    name: IP to my server
    server: ea3b39a3-77a8-4d0b-881d-0bb00a1e7f48
    api_token: xxxxxx

# Request a new floating IPv6 network
- name: Request a floating IP
  cloudscale_ch.cloud.floating_ip:
    name: IPv6 to my server
    ip_version: 6
    prefix_length: 56
    server: 47cec963-fcd2-482f-bdb6-24461b2d47b1
    api_token: xxxxxx
    region: lpg1

# Assign an existing floating network to a different server
- name: Move floating IP to backup server
  cloudscale_ch.cloud.floating_ip:
    ip: '{{ floating_ip.ip }}'
    server: ea3b39a3-77a8-4d0b-881d-0bb00a1e7f48
    api_token: xxxxxx

# Remove a floating IP
- name: Release floating IP
  cloudscale_ch.cloud.floating_ip:
    ip: 192.0.2.123
    state: absent
    api_token: xxxxxx

# Remove a floating IP by name
- name: Release floating IP
  cloudscale_ch.cloud.floating_ip:
    name: IP to my server
    state: absent
    api_token: xxxxxx
'''

RETURN = '''
name:
  description: The name of the floating IP.
  returned: success
  type: str
  sample: my floating ip
  version_added: 1.3.0
href:
  description: The API URL to get details about this floating IP.
  returned: success when state == present
  type: str
  sample: https://api.cloudscale.ch/v1/floating-ips/2001:db8::cafe
network:
  description: The CIDR notation of the network that is routed to your server.
  returned: success
  type: str
  sample: 2001:db8::cafe/128
next_hop:
  description: Your floating IP is routed to this IP address.
  returned: success when state == present
  type: str
  sample: 2001:db8:dead:beef::42
reverse_ptr:
  description: The reverse pointer for this floating IP address.
  returned: success when state == present
  type: str
  sample: 185-98-122-176.cust.cloudscale.ch
server:
  description: The floating IP is routed to this server.
  returned: success when state == present
  type: str
  sample: 47cec963-fcd2-482f-bdb6-24461b2d47b1
ip:
  description: The floating IP address.
  returned: success when state == present
  type: str
  sample: 185.98.122.176
region:
  description: The region of the floating IP.
  returned: success when state == present
  type: dict
  sample: {'slug': 'lpg'}
state:
  description: The current status of the floating IP.
  returned: success
  type: str
  sample: present
tags:
  description: Tags assosiated with the floating IP.
  returned: success
  type: dict
  sample: { 'project': 'my project' }
  version_added: 1.1.0
'''

from ansible.module_utils.basic import AnsibleModule
from ..module_utils.api import (
    AnsibleCloudscaleBase,
    cloudscale_argument_spec,
)


class AnsibleCloudscaleFloatingIp(AnsibleCloudscaleBase):

    def __init__(self, module):
        super(AnsibleCloudscaleFloatingIp, self).__init__(
            module=module,
            resource_key_uuid='network',
            resource_name='floating-ips',
            resource_create_param_keys=[
                'ip_version',
                'server',
                'prefix_length',
                'reverse_ptr',
                'type',
                'region',
                'tags',
            ],
            resource_update_param_keys=[
                'server',
                'reverse_ptr',
                'tags',
            ],
        )
        self.use_tag_for_name = True
        self.query_constraint_keys = ['ip_version']

    def pre_transform(self, resource):
        if 'server' in resource and isinstance(resource['server'], dict):
            resource['server'] = resource['server']['uuid']
        return resource

    def create(self, resource):
        # Fail when missing params for creation
        self._module.fail_on_missing_params(['ip_version', 'name'])
        return super(AnsibleCloudscaleFloatingIp, self).create(resource)

    def get_result(self, resource):
        network = resource.get('network')
        if network:
            self._result['ip'] = network.split('/')[0]
        return super(AnsibleCloudscaleFloatingIp, self).get_result(resource)


def main():
    argument_spec = cloudscale_argument_spec()
    argument_spec.update(dict(
        name=dict(type='str'),
        state=dict(default='present', choices=('present', 'absent'), type='str'),
        network=dict(aliases=('ip',), type='str'),
        ip_version=dict(choices=(4, 6), type='int'),
        server=dict(type='str'),
        type=dict(type='str', choices=('regional', 'global'), default='regional'),
        region=dict(type='str'),
        prefix_length=dict(choices=(56,), type='int'),
        reverse_ptr=dict(type='str'),
        tags=dict(type='dict'),
    ))

    module = AnsibleModule(
        argument_spec=argument_spec,
        required_one_of=(('network', 'name'),),
        supports_check_mode=True,
    )

    cloudscale_floating_ip = AnsibleCloudscaleFloatingIp(module)

    if module.params['state'] == 'absent':
        result = cloudscale_floating_ip.absent()
    else:
        result = cloudscale_floating_ip.present()
    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: organizations_action_batches
short_description: Resource module for organizations _actionbatches
description:
- Manage operations create, update and delete of the resource organizations _actionbatches.
- Create an action batch.
- Delete an action batch.
- Update an action batch.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  actionBatchId:
    description: ActionBatchId path parameter. Action batch ID.
    type: str
  actions:
    description: A set of changes to make as part of this action (<a href='https //developer.cisco.com/meraki/api/#...
      details</a>).
    elements: dict
    suboptions:
      body:
        description: The body of the action.
        type: dict
      operation:
        description: The operation to be used.
        type: str
      resource:
        description: Unique identifier for the resource to be acted on.
        type: str
    type: list
  confirmed:
    description: Set to true for immediate execution. Set to false if the action should
      be previewed before executing. This property cannot be unset once it is true.
      Defaults to false.
    type: bool
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
  synchronous:
    description: Set to true to force the batch to run synchronous. There can be at
      most 20 actions in synchronous batch. Defaults to false.
    type: bool
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for organizations createOrganizationActionBatch
  description: Complete reference of the createOrganizationActionBatch API.
  link: https://developer.cisco.com/meraki/api-v1/#!create-organization-action-batch
- name: Cisco Meraki documentation for organizations deleteOrganizationActionBatch
  description: Complete reference of the deleteOrganizationActionBatch API.
  link: https://developer.cisco.com/meraki/api-v1/#!delete-organization-action-batch
- name: Cisco Meraki documentation for organizations updateOrganizationActionBatch
  description: Complete reference of the updateOrganizationActionBatch API.
  link: https://developer.cisco.com/meraki/api-v1/#!update-organization-action-batch
notes:
  - SDK Method used are
    organizations.Organizations.create_organization_action_batch,
    organizations.Organizations.delete_organization_action_batch,
    organizations.Organizations.update_organization_action_batch,

  - Paths used are
    post /organizations/{organizationId}/actionBatches,
    delete /organizations/{organizationId}/actionBatches/{actionBatchId},
    put /organizations/{organizationId}/actionBatches/{actionBatchId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.organizations_action_batches:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    actions:
    - operation: create
      resource: /devices/QXXX-XXXX-XXXX/switch/ports/3
    confirmed: true
    organizationId: string
    synchronous: true

- name: Update by id
  cisco.meraki.organizations_action_batches:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    actionBatchId: string
    confirmed: true
    organizationId: string
    synchronous: false

- name: Delete by id
  cisco.meraki.organizations_action_batches:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: absent
    actionBatchId: string
    organizationId: string

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "organizationId": "string",
      "confirmed": true,
      "synchronous": true,
      "status": {
        "completed": true,
        "failed": true,
        "errors": [
          "string"
        ],
        "createdResources": [
          {
            "id": "string",
            "uri": "string"
          }
        ]
      },
      "actions": [
        {
          "resource": "string",
          "operation": "string"
        }
      ]
    }
"""
